@@ -62,8 +62,6 @@ typedef struct {
 	GtkWidget	*cmccmd;		/* Entry widget for Click-M-Click command. */
 	GtkWidget	*cmcdelay;		/* HScale widget for cmc delay. */
 
-	GtkWidget	*pathcmd;		/* Entry widget for path-rmb command. */
-
 	GtkWidget	*nonumlock;		/* Ignore numlock check button. */
 
 	MainInfo	*min;			/* This is handy sometimes. */
@@ -124,8 +122,6 @@ static void reset_widgets(P_Controls *page)
 	cmd = ctrl_clickmclick_get_cmdseq(page->ctrlinfo);
 	gtk_entry_set_text(GTK_ENTRY(page->cmccmd), cmd ? cmd : "");
 	gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(page->cmcdelay)), ctrl_clickmclick_get_delay(page->ctrlinfo));
-	cmd = ctrl_general_get_cmdseq(page->ctrlinfo, CTRL_GENERAL_PATH_RMB);
-	gtk_entry_set_text(GTK_ENTRY(page->pathcmd), cmd ? cmd : "");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page->nonumlock), ctrl_numlock_ignore_get(page->ctrlinfo));
 }
 
@@ -222,7 +218,7 @@ static gint evt_kkey_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 		ctrl_key_set_keyname(page->ctrlinfo, key, keyname);
 		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, keyname, -1);
 		gtk_entry_set_text(GTK_ENTRY(page->kdkey), keyname);
-		gtk_signal_emit_stop_by_name(GTK_OBJECT(wid), "key_press_event");
+		g_signal_stop_emission_by_name(G_OBJECT(wid), "key_press_event");
 		page->modified = TRUE;
 	}
 	return TRUE;
@@ -516,35 +512,6 @@ static void evt_clickmclick_delay_changed(GtkAdjustment *adj, gpointer user)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 2004-04-25 -	Track editing of path-rmb command. */
-static void evt_pathcmd_changed(GtkWidget *wid, gpointer user)
-{
-	P_Controls	*page =  user;
-
-	if(page != NULL)
-	{
-		const gchar *text = gtk_entry_get_text(GTK_ENTRY(wid));
-
-		ctrl_general_set_cmdseq(page->ctrlinfo, CTRL_GENERAL_PATH_RMB, text);
-		page->modified = TRUE;
-	}
-}
-
-/* 2004-04-25 -	Help user pick a path-rmb command. */
-static void evt_pathcmd_pick_clicked(GtkWidget *wid, gpointer user)
-{
-	P_Controls	*page = user;
-
-	if(page != NULL)
-	{
-		const gchar *cmd = csq_dialog_sync_new_wait(page->min, ccs_get_current());
-		if(cmd != NULL)
-			gtk_entry_set_text(GTK_ENTRY(page->pathcmd), cmd);
-	}
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 2000-02-18 -	User hit the "Ignore Num Lock?" toggle. Update editing state. */
 static void evt_nonumlock_clicked(GtkWidget *wid, gpointer user)
 {
@@ -640,14 +607,14 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_table_attach(GTK_TABLE(page->kdtable), label, 0, 1, 0, 1,  0,0,0,0);
 	page->kdkey = gtk_entry_new();
 	g_signal_connect(G_OBJECT(page->kdkey), "key_press_event", G_CALLBACK(evt_kkey_press), page);
-	gtk_entry_set_editable(GTK_ENTRY(page->kdkey), FALSE);
+	gtk_editable_set_editable(GTK_EDITABLE(page->kdkey), FALSE);
 	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdkey, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
 	label = gtk_label_new(_("Command"));
 	gtk_table_attach(GTK_TABLE(page->kdtable), label, 0, 1, 1, 2,  0,0,0,0);
 	page->kdcmd = gtk_entry_new();
 	g_signal_connect(G_OBJECT(page->kdcmd), "changed", G_CALLBACK(evt_kcmd_changed), page);
 	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdcmd, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
-	page->kdcmdpick = gui_details_button_new(min->gui->window->window);
+	page->kdcmdpick = gui_details_button_new();
 	g_signal_connect(G_OBJECT(page->kdcmdpick), "clicked", G_CALLBACK(evt_kcmdpick_clicked), page);
 	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdcmdpick, 2, 3, 1, 2,  0,0,0,0);
 	gtk_box_pack_start(GTK_BOX(vbox), page->kdtable, FALSE, FALSE, 0);
@@ -698,7 +665,7 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	page->mdcmd = gtk_entry_new();
 	g_signal_connect(G_OBJECT(page->mdcmd), "changed", G_CALLBACK(evt_mcmd_changed), page);
 	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcmd, 1, 3, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
-	page->mdcmdpick = gui_details_button_new(min->gui->window->window);
+	page->mdcmdpick = gui_details_button_new();
 	g_signal_connect(G_OBJECT(page->mdcmdpick), "clicked", G_CALLBACK(evt_mcmdpick_clicked), page);
 	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcmdpick, 3, 4, 1, 2,  0,0,0,0);
 	gtk_box_pack_start(GTK_BOX(vbox), page->mdtable, FALSE, FALSE, 0);
@@ -720,7 +687,7 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	page->cmccmd = gtk_entry_new();
 	g_signal_connect(G_OBJECT(page->cmccmd), "changed", G_CALLBACK(evt_clickmclick_changed), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->cmccmd, TRUE, TRUE, 0);
-	wid = gui_details_button_new(page->min->gui->window->window);
+	wid = gui_details_button_new();
 	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_clickmclick_pick_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), wid, FALSE, FALSE, 0);
 	label = gtk_label_new(_("Time Limit"));
@@ -734,19 +701,6 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
 	gtk_box_pack_start(GTK_BOX(page->vbox), frame, FALSE, FALSE, 0);
 
-	frame = gtk_frame_new(_("Path Right Click"));
-	hbox = gtk_hbox_new(FALSE, 0);
-	label = gtk_label_new(_("Command"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	page->pathcmd = gtk_entry_new();
-	g_signal_connect(G_OBJECT(page->pathcmd), "changed", G_CALLBACK(evt_pathcmd_changed), page);
-	gtk_box_pack_start(GTK_BOX(hbox), page->pathcmd, TRUE, TRUE, 0);
-	wid = gui_details_button_new(page->min->gui->window->window);
-	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_pathcmd_pick_clicked), page);
-	gtk_box_pack_start(GTK_BOX(hbox), wid, FALSE, FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(frame), hbox);
-	gtk_box_pack_start(GTK_BOX(page->vbox), frame, FALSE, FALSE, 0);
-
 	page->nonumlock = gtk_check_button_new_with_label(_("Ignore Num Lock For All Bindings?"));
 	g_signal_connect(G_OBJECT(page->nonumlock), "clicked", G_CALLBACK(evt_nonumlock_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->nonumlock, FALSE, FALSE, 0);
