@@ -9,10 +9,12 @@ extern gboolean	copy_link(MainInfo *min, const gchar *from, const gchar *to, con
 extern gboolean copy_fifo(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat);
 extern gboolean	copy_socket(MainInfo *min, const gchar *from, const gchar *to, const struct stat *sstat);
 
-/* Toplevel copying functions, for external use. */
+/* Top-level copying functions, for external use. */
 extern gboolean	copy_dir(MainInfo *min, const gchar *from, const gchar *to);
 extern gboolean	copy_file(MainInfo *min, const gchar *from, const gchar *to, const struct stat *sstat);
 
+extern gboolean	copy_gfile(MainInfo *min, DirPane *src, DirPane *dst, GFile *from, GFile *to, GError **err);
+
 extern gint	cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca);
 
 extern void	cfg_copy(MainInfo *min);
