@@ -10,7 +10,6 @@
 #include <fcntl.h>
 
 #include "cmd_delete.h"
-#include "convutil.h"
 #include "dialog.h"
 #include "dirpane.h"
 #include "errors.h"
@@ -28,25 +27,24 @@
 /* 2000-02-12 -	Do the actual joining of the selected files. Returns TRUE on success. */
 static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkListStore *store, GtkEntry *entry, gsize total_size)
 {
-	gchar		buf[PATH_MAX], *file;
-	const gchar	*str;
-	gsize		done_size = 0U;
-	OvwRes		owres;
-	gboolean	doit = TRUE;
-	Conv		conv;
+	const gchar		*str = gtk_entry_get_text(entry);
+	GFile			*out;
+	OvwRes			owres;
+	gboolean		doit = TRUE;
+	GError			*err = NULL;
+	GFileOutputStream	*outs;
+	gsize			done_size = 0;
 
-	/* Protect against empty destination name. */
-	if((str = gtk_entry_get_text(entry)) == NULL || *str == '\0')
+	/* Protect against slashes in destination name. No rename-trickery! */
+	if(g_utf8_strchr(str, -1, G_DIR_SEPARATOR) != NULL)
 	{
 		err_set(min, EINVAL, CMD_ID, NULL);
 		return FALSE;
 	}
 
-	file = conv_begin(&conv, str);
-
+	out = dp_get_file_from_name_display(dst, str);
 	ovw_overwrite_begin(min, _("\"%s\" Already Exists - Continue With Join?"), 0U);
-	g_snprintf(buf, sizeof buf, "%s/%s", dst->dir.path, file);
-	owres = ovw_overwrite_file(min, buf, NULL);
+	owres = ovw_overwrite_unary_file(dst, out);
 	switch(owres)
 	{
 	case OVW_SKIP:
@@ -56,68 +54,73 @@ static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkListStore
 	case OVW_PROCEED:
 		break;
 	case OVW_PROCEED_FILE:
-		doit = del_delete_file(min, buf);
-		break;
 	case OVW_PROCEED_DIR:
-		doit = del_delete_dir(min, buf, FALSE);
+		doit = del_delete_gfile(min, out, FALSE, &err);
 		break;
 	}
-	if(doit)
+	if(!doit)
 	{
-		gint	fout;
-
-		if((fout = open(buf, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) >= 0)
-		{
-			GtkTreeIter	iter;
-			gint		fin;
-			DirRow		*dr;
-			gsize		put;
-
-			pgs_progress_begin(min, _("Joining..."), PFLG_BYTE_VISIBLE);
-			gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-			while(gtk_list_store_iter_is_valid(store, &iter))
-			{
-				gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 2, &dr, -1);
-				if((fin = open(dp_full_name(src, DP_ROW_INDEX(src, dr)), O_RDONLY)) >= 0)
-				{
-					pgs_progress_item_begin(min, DP_ROW_NAME(dr), DP_ROW_STAT(dr).st_size);
-					put = fut_copy(fin, fout, 256 << 10);
-					pgs_progress_item_end(min);
-					if(close(fin) != 0)
-					{
-						err_set(min, errno, CMD_ID, DP_ROW_NAME(dr));
-						break;
-					}
-					if(put != DP_ROW_STAT(dr).st_size)
-					{
-						err_set(min, 000, CMD_ID, DP_ROW_NAME(dr));
-						break;
-					}
-					done_size += put;
-					dp_unselect(src, DP_ROW_INDEX(src, dr));
-				}
-				else
-				{
-					err_set(min, errno, CMD_ID, DP_ROW_NAME(dr));
-					break;
-				}
-				if(!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
-					break;
-			}
-			pgs_progress_end(min);
-			close(fout);
-			if(done_size == total_size)
-				dp_rescan_post_cmd(dst);
-			else
-				unlink(buf);
-		}
-		else
-			err_set(min, errno, CMD_ID, buf);
+		err_set_gerror(min, &err, CMD_ID, out);
+		g_object_unref(out);
+		return FALSE;
 	}
-	ovw_overwrite_end(min);
-	conv_end(&conv);
 
-	return doit ? done_size == total_size : TRUE;
+	if((outs = g_file_create(out, G_FILE_CREATE_NONE, NULL, &err)) != NULL)
+	{
+		GtkTreeIter		iter;
+		gsize			put;
+
+		pgs_progress_begin(min, _("Joining..."), PFLG_BYTE_VISIBLE);
+		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+		while(gtk_list_store_iter_is_valid(store, &iter))
+		{
+			GFile	*in;
+			DirRow2	*dr;
+
+			gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 2, &dr, -1);
+			if((in = dp_get_file_from_row(src, dr)) != NULL)
+			{
+				GFileInputStream	*ins;
+
+				if((ins = g_file_read(in, NULL, &err)) != NULL)
+				{
+					pgs_progress_item_begin(min, dp_row_get_name_display(dp_get_tree_model(src), dr), dp_row_get_size(dp_get_tree_model(src), dr));
+					put = g_output_stream_splice(G_OUTPUT_STREAM(outs), G_INPUT_STREAM(ins), G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE, NULL, &err);
+					if(put < 0 || !g_input_stream_close(G_INPUT_STREAM(ins), NULL, &err))
+						break;
+					pgs_progress_item_end(min);
+					dp_unselect(src, dr);
+					done_size += put;
+				}
+				g_object_unref(in);
+			}
+			else
+				break;
+			if(!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
+				break;
+		}
+		pgs_progress_end(min);
+		if(done_size == total_size)
+			dp_rescan_post_cmd(dst);
+	}
+	if(done_size != total_size)
+		err_set_gerror(min, &err, CMD_ID, out);
+
+	g_object_unref(out);
+	ovw_overwrite_end(min);
+
+	return done_size == total_size;
+}
+
+static void evt_dest_changed(GtkEditable *w, gpointer user)
+{
+	Dialog		*dlg = user;
+	const gchar	*str = gtk_entry_get_text(GTK_ENTRY(w));
+	gsize		len;
+
+	/* Protect against empty destination name, nicely. */
+	len = strlen(str);
+	dlg_dialog_set_positive_enabled(dlg, len > 0);
 }
 
 gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
@@ -129,15 +132,18 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 	if((sel = dp_get_selection(src)) != NULL)
 	{
-		GSList	*iter;
-		gsize	tot = 0, num = 0;
+		GtkTreeModel	*m = dp_get_tree_model(src);
+		GSList		*iter;
+		gsize		tot = 0, num = 0;
 
-		/* Compute size of all regular files in the selection. */
+		/* Compute size of all regular files in the selection; ignore 0-sized files. */
 		for(iter = sel; iter != NULL; iter = g_slist_next(iter))
 		{
-			if(S_ISREG(DP_ROW_STAT((DirRow *) iter->data).st_mode))
+			guint64	size = dp_row_get_size(m, iter->data);
+			/* Follow symlinks, since we want to allow the joining of a bunch of link targets. */
+			if(size > 0 && dp_row_get_file_type(m, iter->data, TRUE) == G_FILE_TYPE_REGULAR)
 			{
-				tot += DP_ROW_STAT((DirRow *) iter->data).st_size;
+				tot += size;
 				num++;
 			}
 		}
@@ -148,15 +154,14 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 		if(num > 1)
 		{
 			Dialog			*dlg;
-			GtkWidget		*scwin, *vbox, *label, *entry;
+			GtkWidget		*scwin, *vbox, *label, *entry, *view;
 			GtkListStore		*store;
-			GtkWidget		*view;
 			GtkCellRenderer		*cr, *sr;
 			GtkTreeViewColumn	*vc;
 			GtkTreeSelection	*treesel;
 			gchar			buf[FILENAME_MAX + 64];
-			const gchar		*first = NULL;
-			gint			len;
+			const gchar		*first = NULL, *tail;
+			gunichar		ch;
 
 			vbox  = gtk_vbox_new(FALSE, 0);
 			label = gtk_label_new(_("Click and Drag Files to Reorder, Then Click Join."));
@@ -166,7 +171,7 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 			{
 				gchar	sbuf[16];
 
-				sze_put_size(sbuf, sizeof sbuf, tot, SZE_AUTO);
+				sze_put_offset(sbuf, sizeof sbuf, tot, SZE_AUTO, 3, ',');
 				g_snprintf(buf, sizeof buf, _("The total size is %s (%lu bytes)."), sbuf, (unsigned long) tot);
 			}
 			else
@@ -181,17 +186,17 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 			store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_POINTER);
 			for(iter = sel; iter != NULL; iter = g_slist_next(iter))
 			{
-				if(S_ISREG(DP_ROW_STAT((DirRow *) iter->data).st_mode))
+				if(dp_row_get_file_type(m, iter->data, TRUE) == G_FILE_TYPE_REGULAR && dp_row_get_size(m, iter->data) > 0)
 				{
 					GtkTreeIter	ti;
 
 					gtk_list_store_append(store, &ti);
-					gtk_list_store_set(store, &ti, 0, DP_ROW_NAME((DirRow *) iter->data),
-								       1, DP_ROW_STAT((DirRow *) iter->data).st_size,
+					gtk_list_store_set(store, &ti, 0, dp_row_get_name_display(m, iter->data),
+								       1, dp_row_get_size(m, iter->data),
 								       2, iter->data,
 								       -1);
 					if(first == NULL)
-						first = DP_ROW_NAME((DirRow *) iter->data);
+						first = dp_row_get_name_display(m, iter->data);
 				}
 			}
 
@@ -214,22 +219,21 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 			label = gtk_label_new(_("Enter Destination File Name"));
 			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
-			entry = gtk_entry_new_with_max_length(FILENAME_MAX - 1);
-			/* Heuristic: remove trailing hexadecimal number and period, and position cursor before last remaining period. */
-			if((len = g_snprintf(buf, sizeof buf, "%s", first)) > 0)
+			entry = gtk_entry_new();
+			gtk_entry_set_max_length(GTK_ENTRY(entry), FILENAME_MAX - 1);
+			/* Get a pointer to the terminating '\0' character. */
+			tail = g_utf8_offset_to_pointer(first, g_utf8_strlen(first, -1));
+			do
 			{
-				const gchar	*dot;
-				int	i;
-
-				for(i = len - 1; i > 0 && (isxdigit(buf[i]) || buf[i] == '.'); i--)
-					buf[i] = '\0';
-				gtk_entry_set_text(GTK_ENTRY(entry), buf);
-				if((dot = strrchr(buf, '.')) != NULL)
-					gtk_entry_set_position(GTK_ENTRY(entry), dot - buf);
-			}
+				tail = g_utf8_find_prev_char(first, tail);	/* Step backward. */
+				ch = g_utf8_get_char(tail);
+			} while(g_unichar_isxdigit(ch) || ch == '.');
+			gtk_entry_buffer_set_text(gtk_entry_get_buffer(GTK_ENTRY(entry)), first, g_utf8_pointer_to_offset(first, tail) + 1);
+			gtk_editable_set_position(GTK_EDITABLE(entry), -1);
 			gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
 
 			dlg = dlg_dialog_sync_new(vbox, _("Join"), _("_Join|_Cancel"));
+			g_signal_connect(G_OBJECT(entry), "changed", G_CALLBACK(evt_dest_changed), dlg);
 			gtk_widget_show_all(vbox);
 			gtk_widget_grab_focus(entry);
 			if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
@@ -238,6 +242,5 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 		}
 		dp_free_selection(sel);
 	}
-	err_show(min);
 	return ret;
 }
