@@ -14,7 +14,6 @@
 #include "cmdseq.h"
 #include "cmdseq_config.h"
 #include "configure.h"
-#include "convutil.h"
 #include "dialog.h"
 #include "dirpane.h"
 #include "errors.h"
@@ -45,70 +44,64 @@ static CmdCfg	*mkdir_cmc = NULL;
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-08-27 -	This gets called when user closes down the mkdir dialog. Create the
-**		dir (and, optionally, enter or focus it).
-*/
-static void make_dir(MkdInfo *mdi)
+static gboolean make_dir(const MkdInfo *mdi)
 {
-	MainInfo	*min = mdi->src->main;
-	gchar		buf[PATH_MAX], *name;
-	const gchar	*text = gtk_entry_get_text(GTK_ENTRY(mdi->entry));
-	gboolean	doit = FALSE;
-	struct stat	stbuf;
-	Conv		conv;
+	const gchar	*text;
+	GFile		*dest;
+	gboolean	ok = FALSE;
+	GError		*err = NULL;
 
-	if(*text == '\0')				/* Avoid attempting to create "". */
-		return;
-	if(*mdi->src->dir.path == '\0')			/* Don't blow up in strlen() below. */
-		return;
-	name = conv_begin(&conv, text);
-	if(strchr(name, G_DIR_SEPARATOR) != NULL)	/* Also avoid clever paths. */
-	{
-		conv_end(&conv);
-		return;
-	}
+	if(mdi == NULL)
+		return FALSE;
+	text = gtk_entry_get_text(GTK_ENTRY(mdi->entry));
+	if(*text == '\0')
+		return FALSE;
+	if(mdi->src->dir.root == NULL)
+		return FALSE;
 
-	/* Build destination directory name, taking care to get a single slash between parts. */
-	g_snprintf(buf, sizeof buf, "%s%s%s", mdi->src->dir.path,
-			mdi->src->dir.path[strlen(mdi->src->dir.path) - 1] == G_DIR_SEPARATOR ? "" : G_DIR_SEPARATOR_S,
-			name);
-	ovw_overwrite_begin(min, _("\"%s\" Already Exists - Proceed With MkDir?"), 0UL);
-	switch(ovw_overwrite_file(min, buf, NULL))
+	err_clear(mdi->src->main);
+	if((dest = g_file_get_child_for_display_name(mdi->src->dir.root, text, &err)) != NULL)
 	{
-	case OVW_SKIP:
-	case OVW_CANCEL:
-		break;
-	case OVW_PROCEED:
-		doit = TRUE;
-		break;
-	case OVW_PROCEED_FILE:
-		doit = del_delete_file(min, buf);
-		break;
-	case OVW_PROCEED_DIR:
-		doit = FALSE;
-	}
-	ovw_overwrite_end(min);
-	err_clear(min);
-	if(doit)
-		mkdir(buf, 0755);
-	if(lstat(buf, &stbuf) == 0 && S_ISDIR(stbuf.st_mode))
-	{
-		if(mkdir_options.cd_new)
-			csq_execute_format(min, "DirEnter 'dir=%s'", stu_escape(buf));
-		else
+		ovw_overwrite_begin(mdi->src->main, _("\"%s\" Already Exists - Proceed With MkDir?"), 0UL);
+		switch(ovw_overwrite_unary_file(mdi->src, dest))
 		{
-			dp_rescan(mdi->src);
-			csq_execute_format(min, "DpGotoRow focus=%s nocase=no 're=%s'", mkdir_options.focus_new ? "yes" : "no", name);
+		case OVW_SKIP:
+		case OVW_CANCEL:
+			ok = FALSE;
+			break;
+		case OVW_PROCEED:
+			ok = TRUE;
+			break;
+		case OVW_PROCEED_FILE:
+			ok = del_delete_gfile(mdi->src->main, dest, FALSE, &err);
+			break;
+		case OVW_PROCEED_DIR:
+			ok = FALSE;
+			break;
 		}
+		ovw_overwrite_end(mdi->src->main);
+		if(ok)
+		{
+			ok = g_file_make_directory(dest, NULL, &err);
+			if(ok)
+			{
+				gchar	*pn = g_file_get_parse_name(dest);
+
+				if(mkdir_options.cd_new)
+					ok = csq_execute_format(mdi->src->main, "DirEnter 'dir=%s'", pn);
+				else if(mkdir_options.focus_new)
+					ok = csq_execute_format(mdi->src->main, "DpGotoRow focus=%s nocase=no 're=%s'", mkdir_options.focus_new ? "yes" : "no", pn);
+				if(!mkdir_options.cd_new)
+					dp_rescan_post_cmd(mdi->src);
+				g_free(pn);
+			}
+		}
+		g_object_unref(dest);
 	}
-	else
-	{
-		err_set(min, errno, CMD_ID, buf);
-		err_show(min);
-	}
-	conv_end(&conv);
-	if(mkdir_options.modified)
-		cfg_modified_set(min);
+	if(err != NULL)
+		err_set_gerror(mdi->src->main, &err, text, NULL);
+
+	return ok;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -119,12 +112,14 @@ gint cmd_mkdir(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	Dialog		*dlg;
 	MkdInfo		mi;
 	GtkWidget	*wid;
+	gboolean	ok;
 
 	mi.src = src;
 	mi.vbox = gtk_vbox_new(FALSE, 0);
 	wid = gtk_label_new(_("Enter Name of Directory to Create"));
 	gtk_box_pack_start(GTK_BOX(mi.vbox), wid, FALSE, FALSE, 0);
-	mi.entry = gtk_entry_new_with_max_length(MAXNAMLEN - 1);
+	mi.entry = gtk_entry_new();
+	gtk_entry_set_max_length(GTK_ENTRY(mi.entry), MAXNAMLEN - 1);
 	gtk_box_pack_start(GTK_BOX(mi.vbox), mi.entry, FALSE, FALSE, 0);
 	wid = cmc_field_build(mkdir_cmc, "cd_new", &mkdir_options);
 	gtk_box_pack_start(GTK_BOX(mi.vbox), wid, FALSE, FALSE, 0);
@@ -133,10 +128,10 @@ gint cmd_mkdir(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	dlg = dlg_dialog_sync_new(mi.vbox, _("Make Directory"), NULL);
 	gtk_widget_grab_focus(mi.entry);
 	if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
-		make_dir(&mi);
+		ok = make_dir(&mi);
 	dlg_dialog_sync_destroy(dlg);
 
-	return 1;
+	return ok;
 }
 
 /* ----------------------------------------------------------------------------------------- */
