@@ -16,14 +16,11 @@
 #include "gentoo.h"
 
 #include <ctype.h>
-#include <regex.h>
 
 #include "cmd_delete.h"
-#include "convutil.h"
 #include "dialog.h"
 #include "dirpane.h"
 #include "errors.h"
-#include "fileutil.h"
 #include "overwrite.h"
 #include "strutil.h"
 
@@ -75,106 +72,84 @@ typedef struct {
 
 /* ----------------------------------------------------------------------------------------- */
 
-static gint do_rename(MainInfo *min, DirPane *dp, GSList **iter, GString *nn, gint *err)
+static gboolean do_rename(MainInfo *min, DirPane *dp, const DirRow2 *row, const gchar *newname, GError **error)
 {
-	if(strcmp(DP_SEL_NAME(*iter), nn->str) != 0)
-	{
-		OvwRes	ores;
+	OvwRes		ores;
+	GFile		*file;
+	gboolean	ok, doit = TRUE;
 
-		ores = ovw_overwrite_file(min, nn->str, NULL);
-		if(ores == OVW_SKIP)
-			return 0;
-		else if(ores == OVW_CANCEL)
+	if(strcmp(dp_row_get_name_display(dp_get_tree_model(dp), row), newname) == 0)
+	{
+		dp_unselect(dp, row);
+		return TRUE;
+	}
+
+	/* Get imaginary destination file. */
+	if((file = dp_get_file_from_name(dp, newname)) == NULL)
+		return FALSE;
+	ores = ovw_overwrite_unary_file(dp, file);
+	if(ores == OVW_SKIP)
+		ok = !(doit = FALSE);	/* H4xX0r. */
+	else if(ores == OVW_CANCEL)
+		ok = doit = FALSE;
+	else if(ores == OVW_PROCEED_FILE || ores == OVW_PROCEED_DIR)
+		ok = del_delete_gfile(min, file, FALSE, error);
+	else
+		ok = TRUE;
+
+	if(ok && doit)
+	{
+		GFile	*dfile;
+
+		if((dfile = dp_get_file_from_row(dp, row)) != NULL)
 		{
-			*iter = NULL;
-			return -1;
-		}
-		else if(ores == OVW_PROCEED_FILE)
-		{
-			if(!del_delete_file(min, nn->str))
-			{
-				*iter = NULL;
-				return -1;
-			}
-		}
-		else if(ores == OVW_PROCEED_DIR)
-		{
-			if(!del_delete_dir(min, nn->str, FALSE))
-			{
-				*iter = NULL;
-				return -1;
-			}
-		}
-		if(rename(DP_SEL_NAME(*iter), nn->str) != 0)
-		{
-			*err = errno;
-			dp_rescan_post_cmd(dp);
-			err_set(min, *err, CMD_ID, DP_SEL_NAME(*iter));
+			GFile	*nfile;
+
+			if((nfile = g_file_set_display_name(dfile, newname, NULL, error)) != NULL)
+				g_object_unref(nfile);
+			g_object_unref(dfile);
+			ok = nfile != NULL;
 		}
 		else
-			dp_unselect(dp, DP_SEL_INDEX(dp, *iter));
+			ok = FALSE;
 	}
-	return errno;
+	if(!ok && error != NULL && *error != NULL)
+		err_set_gerror(min, error, CMD_ID, file);
+	g_object_unref(file);
+
+	if(ok)
+		dp_unselect(dp, row);
+
+	return ok;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-static gint rename_simple(RenREInfo *ri)
+static gboolean rename_simple(RenREInfo *ri, GError **err)
 {
-	const gchar	*fromd, *tod, *from, *to, *ptr, *base;
-	gint		err = 0;
-	gsize		flen;
+	const gchar	*fromd, *tod;
 	GString		*nn;
 	GSList		*iter;
-	gboolean	nocase, global;
-	Conv		conv;
+	gboolean	nocase, global, ok = TRUE;
 
 	if((fromd = gtk_entry_get_text(GTK_ENTRY(ri->r_from))) == NULL)
-		return -1;
-
+		return 0;
 	tod = gtk_entry_get_text(GTK_ENTRY(ri->r_to));
 
 	nocase = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->r_cnocase));
 	global = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->r_cglobal));
 
-	from = conv_begin(&conv, fromd);
-	to = conv_add(&conv, tod);
-
-	flen = strlen(from);
-
-	to = gtk_entry_get_text(GTK_ENTRY(ri->r_to));
-
 	ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
 	nn = g_string_new("");
-	for(iter = ri->selection; !err && iter != NULL; iter = g_slist_next(iter))
+	for(iter = ri->selection; ok && (iter != NULL); iter = g_slist_next(iter))
 	{
-		g_string_truncate(nn, 0);
-
-		base = DP_SEL_NAME(iter);
-		while((ptr = nocase ? stu_strcasechr(base, *from) : strchr(base, *from)) != NULL)
-		{
-			for(; base < ptr; base++)
-				g_string_append_c(nn, *base);
-			if((nocase ? g_strncasecmp(ptr, from, flen) : strncmp(ptr, from, flen)) == 0)
-			{
-				g_string_append(nn, to);
-				base += flen;
-				if(!global)
-					break;
-			}
-			else
-				g_string_append_c(nn, *base++);
-		}
-		g_string_append(nn, base);
-		if(strchr(nn->str, G_DIR_SEPARATOR) != NULL)
-			continue;
-		do_rename(ri->min, ri->src, &iter, nn, &err);
+		if(stu_replace_simple(nn, dp_row_get_name_display(dp_get_tree_model(ri->src), iter->data), fromd, tod, global, nocase) > 0)
+			ok = do_rename(ri->min, ri->src, iter->data, nn->str, err);
 	}
 	g_string_free(nn, TRUE);
 	ovw_overwrite_end(ri->min);
-	conv_end(&conv);
 
-	return err;
+	return ok;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -184,25 +159,36 @@ static gint rename_simple(RenREInfo *ri)
 **		by <match>. To get a single dollar in the output, write $$ in <def>.
 **		Returns TRUE if the entire <def> was successfully interpolated, FALSE on error.
 */
-static gboolean interpolate(GString *str, const gchar *def, const gchar *src, const regmatch_t *match)
+static gboolean interpolate(GString *str, const gchar *def, const GMatchInfo *match)
 {
 	const gchar	*ptr;
 
 	g_string_truncate(str, 0);
-	for(ptr = def; *ptr;)
+	for(ptr = def; *ptr; ptr = g_utf8_next_char(ptr))
 	{
-		if(*ptr == '$')
-		{
-			ptr++;
-			if(isdigit((guchar) *ptr))
-			{
-				gint	slot, i;
+		gunichar	here;
 
-				slot = *ptr++ - '0';
-				if((i = match[slot].rm_so) != -1)
+		here = g_utf8_get_char(ptr);
+		if(here == '$')
+		{
+			ptr = g_utf8_next_char(ptr);
+			here = g_utf8_get_char(ptr);
+			if(g_unichar_isdigit(here))
+			{
+				gint	slot;
+
+				slot = g_unichar_digit_value(here);
+				if(slot >= 0 && slot <= 9)
 				{
-					for(; i < match[slot].rm_eo; i++)
-						g_string_append_c(str, src[i]);
+					gchar	*ms = g_match_info_fetch(match, slot);
+
+					if(ms != NULL)
+					{
+						g_string_append(str, ms);
+						g_free(ms);
+					}
+					else
+						return FALSE;
 				}
 				else
 					return FALSE;
@@ -213,95 +199,82 @@ static gboolean interpolate(GString *str, const gchar *def, const gchar *src, co
 				return FALSE;
 		}
 		else
-			g_string_append_c(str, *ptr++);
+			g_string_append_unichar(str, here);
 	}
 	return TRUE;
 }
 
-static gint rename_regexp(RenREInfo *ri)
+static gboolean rename_regexp(RenREInfo *ri, GError **err)
 {
-	const gchar	*fromdefd, *tod, *fromdef, *to;
-	gint		reerr, reflags, err = 0;
-	regex_t		fromre;
-	Conv		conv;
+	const gchar	*fromdefd, *tod;
+	guint		reflags;
+	GRegex		*fromre = NULL;
+	gboolean	ok = TRUE;
 
 	if((fromdefd = gtk_entry_get_text(GTK_ENTRY(ri->re_from))) == NULL)
-		return -1;
+		return FALSE;
 	if((tod = gtk_entry_get_text(GTK_ENTRY(ri->re_to))) == NULL)
-		return -1;
+		return FALSE;
 
-	fromdef = conv_begin(&conv, fromdefd);
-	to = conv_add(&conv, tod);
-
-	reflags = REG_EXTENDED | gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->re_cnocase));
-	if((reerr = regcomp(&fromre, fromdef, reflags)) == 0)
+	reflags = G_REGEX_EXTENDED | (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->re_cnocase)) ? G_REGEX_CASELESS : 0);
+	if((fromre = g_regex_new(fromdefd, reflags, G_REGEX_MATCH_NOTEMPTY, err)) != NULL)
 	{
 		GSList		*iter;
 		GString		*nn;
-		regmatch_t	match[10];
 
 		ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
 		nn = g_string_new("");
-		for(iter = ri->selection; iter != NULL; iter = g_slist_next(iter))
+		for(iter = ri->selection; ok && (iter != NULL); iter = g_slist_next(iter))
 		{
-			if(regexec(&fromre, DP_SEL_NAME(iter), sizeof match / sizeof match[0], match, 0) == 0)
+			GMatchInfo	*mi = NULL;
+
+			if(g_regex_match(fromre, dp_row_get_name_display(dp_get_tree_model(ri->src), iter->data), G_REGEX_MATCH_NOTEMPTY, &mi))
 			{
-				if(interpolate(nn, to, DP_SEL_NAME(iter), match) && strchr(nn->str, G_DIR_SEPARATOR) == NULL)
-					do_rename(ri->min, ri->src, &iter, nn, &err);
+				if(interpolate(nn, tod, mi) && g_utf8_strchr(nn->str, G_DIR_SEPARATOR, -1) == NULL)
+					ok = do_rename(ri->min, ri->src, iter->data, nn->str, err);
+				g_match_info_free(mi);
 			}
 		}
 		g_string_free(nn, TRUE);
 		ovw_overwrite_end(ri->min);
 	}
-	else
-	{
-		gchar	buf[128];
-
-		strcpy(buf, _("Regular expression error:\n"));
-		regerror(reerr, &fromre, buf + strlen(buf), sizeof buf - strlen(buf) - 1);
-		dlg_dialog_async_new_error(buf);
-	}
-	conv_end(&conv);
-
-	return err;
+	return ok;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-static int rename_map(RenREInfo *ri)
+static gboolean rename_map(RenREInfo *ri, GError **err)
 {
-	const char	*fromdef, *todef, *remdef, *map;
+	const gchar	*fromdef, *todef, *remdef;
 	GSList		*iter;
 	GString		*nn, *nr;
-	gint		err = 0;
 	gsize		fl, tl, rl, i, nl;
-	Conv		conv;
+	gboolean	ok = TRUE;
 
 	if((fromdef = gtk_entry_get_text(GTK_ENTRY(ri->map_from))) == NULL)
-		return -1;
+		return FALSE;
 	if((todef = gtk_entry_get_text(GTK_ENTRY(ri->map_to))) == NULL)
-		return -1;
-	if(strchr(todef, G_DIR_SEPARATOR) != NULL)	/* As always, prevent rogue moves. */
-		return -1;
+		return FALSE;
+	if(g_utf8_strchr(todef, -1, G_DIR_SEPARATOR) != NULL)	/* As always, prevent rogue moves. */
+		return FALSE;
 	if((remdef = gtk_entry_get_text(GTK_ENTRY(ri->map_remove))) == NULL)
-		return -1;
+		return FALSE;
 
+	/* Totally important: some lengths are in chars (for iteration), some in bytes for g_utf8_strchr(). */
 	fl = strlen(fromdef);
-	tl = strlen(todef);
+	tl = g_utf8_strlen(todef, -1);
 	rl = strlen(remdef);
 
 	ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
 	nn = g_string_new("");
 	nr = g_string_new("");
-	conv_init(&conv);
-	conv_add(&conv, "");	/* Pre-load the Conv, so we can use set() below. */
 	for(iter = ri->selection; iter != NULL; iter = g_slist_next(iter))
 	{
-		const char	*ptr;
+		const gchar	*ptr, *map;
 
 		/* Step 1: clear the dynamic string. */
 		g_string_truncate(nn, 0);
-		ptr = DP_SEL_NAMED(iter);
+		ptr = dp_row_get_name_display(dp_get_tree_model(ri->src), iter->data);
 		nl = g_utf8_strlen(ptr, -1);
 		/* Step 2: go through and do the mapping. */
 		for(i = 0; i < nl; i++, ptr = g_utf8_next_char(ptr))
@@ -310,7 +283,7 @@ static int rename_map(RenREInfo *ri)
 
 			if((map = g_utf8_strchr(fromdef, fl, ch)) != NULL)
 			{
-				glong	index = g_utf8_pointer_to_offset(fromdef, map);
+				glong		index = g_utf8_pointer_to_offset(fromdef, map);
 				const gchar	*tptr = g_utf8_offset_to_pointer(todef, index);
 
 				ch = g_utf8_get_char(tptr);	/* Map. */
@@ -328,23 +301,20 @@ static int rename_map(RenREInfo *ri)
 			if(g_utf8_strchr(remdef, rl, ch) == NULL)
 				g_string_append_unichar(nr, ch);
 		}
-		/* Step 4: re-use the 'nn' string with filesystem version of final string. */
-		g_string_truncate(nn, 0);
-		g_string_assign(nn, conv_set(&conv, 0, nr->str));
-		do_rename(ri->min, ri->src, &iter, nn, &err);
+		/* Step 4: do the rename, using the now-built "display name". */
+		ok = do_rename(ri->min, ri->src, iter->data, nr->str, err);
 	}
-	conv_end(&conv);
 	g_string_free(nr, TRUE);
 	g_string_free(nn, TRUE);
 	ovw_overwrite_end(ri->min);
 
-	return err;
+	return ok;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
 /* 2008-07-19 -	Converts the given string to have an upper-case initial character, while the
- *		the remainder are lower-case. This is a very western idea of a useful case
+ *		remainder is lower-case. This is a very western idea of a useful case
  *		conversion, but ... I'll include it anyway.
 */
 static gchar * utf8_str_initial(const gchar *str, gssize len)
@@ -377,16 +347,13 @@ static gchar * utf8_str_initial(const gchar *str, gssize len)
  *		using UTF-8 ... So we use the display versions of the names, downcase in UTF-8,
  *		and then convert back.
 */
-static int rename_case(RenREInfo *ri)
+static gboolean rename_case(RenREInfo *ri, GError **err)
 {
 	GString		*nn;
 	GSList		*iter;
-	gint		err = 0;
-	Conv		conv;
 	gchar *		(*func)(const gchar *, gssize), *cc;
+	gboolean	ok = TRUE;
 
-	ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
-	nn = g_string_new("");
 	/* Select the case-conversion function, once. */
 	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->case_lower)))
 		func = g_utf8_strdown;
@@ -395,29 +362,34 @@ static int rename_case(RenREInfo *ri)
 	else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->case_initial)))
 		func = utf8_str_initial;
 	else
-		func = NULL;
-	for(iter = ri->selection; iter != NULL; iter = g_slist_next(iter))
+		return FALSE;
+
+	ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
+	nn = g_string_new("");
+	for(iter = ri->selection; ok && (iter != NULL); iter = g_slist_next(iter))
 	{
+		const gchar	*dn = dp_row_get_name_display(dp_get_tree_model(ri->src), iter->data);
+
 		/* Step 1: set the dynamic string to the input filename. */
-		g_string_assign(nn, DP_SEL_NAMED(iter));
+		g_string_assign(nn, dn);
 		/* Step 2: perform case conversion. */
 		cc = func(nn->str, nn->len);
-		g_string_assign(nn, conv_begin(&conv, cc));
-		g_free(cc);
 		/* Step 3: if resulting name is different, rename on disk. */
-		if(strcmp(DP_SEL_NAME(iter), nn->str) != 0)
-			do_rename(ri->min, ri->src, &iter, nn, &err);
-		conv_end(&conv);
+		if(strcmp(dn, cc) != 0)
+		{
+			ok = do_rename(ri->min, ri->src, iter->data, cc, err);
+		}
+		g_free(cc);
 	}
 	g_string_free(nn, TRUE);
 	ovw_overwrite_end(ri->min);
 
-	return err;
+	return ok;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-static void evt_nbook_switchpage(GtkWidget *wid, GtkNotebookPage *page, gint page_num, gpointer user)
+static void evt_nbook_switchpage(GtkWidget *wid, gpointer page, gint page_num, gpointer user)
 {
 	RenREInfo	*ri = user;
 
@@ -429,6 +401,7 @@ gint cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GtkWidget		*label, *table;
 	static RenREInfo	ri = { NULL };
+	gboolean		ok = TRUE;
 
 	ri.min	= min;
 	ri.src	= src;
@@ -518,9 +491,9 @@ gint cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 		gtk_box_pack_start(GTK_BOX(ri.case_vbox), label, FALSE, FALSE, 0);
 		ri.case_lower = gtk_radio_button_new_with_label(NULL, _("Lower Case?"));
 		gtk_box_pack_start(GTK_BOX(ri.case_vbox), ri.case_lower, FALSE, FALSE, 0);
-		ri.case_upper = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(ri.case_lower)), _("Upper Case?"));
+		ri.case_upper = gtk_radio_button_new_with_label(gtk_radio_button_get_group(GTK_RADIO_BUTTON(ri.case_lower)), _("Upper Case?"));
 		gtk_box_pack_start(GTK_BOX(ri.case_vbox), ri.case_upper, FALSE, FALSE, 0);
-		ri.case_initial = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(ri.case_lower)), _("Upper Case Initial?"));
+		ri.case_initial = gtk_radio_button_new_with_label(gtk_radio_button_get_group(GTK_RADIO_BUTTON(ri.case_lower)), _("Upper Case Initial?"));
 		gtk_box_pack_start(GTK_BOX(ri.case_vbox), ri.case_initial, FALSE, FALSE, 0);
 		gtk_widget_show_all(ri.map_vbox);
 		gtk_notebook_append_page(GTK_NOTEBOOK(ri.nbook), ri.case_vbox, gtk_label_new(_("Case")));
@@ -529,36 +502,26 @@ gint cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 		ri.page = 0;
 	}
-	gtk_notebook_set_page(GTK_NOTEBOOK(ri.nbook), ri.page);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(ri.nbook), ri.page);
 	gtk_widget_grab_focus(ri.page ? ri.re_from : ri.r_from);
 
 	if(dlg_dialog_sync_wait(ri.dlg) == DLG_POSITIVE)
 	{
-		gchar	old[PATH_MAX];
-		gint	err = 0;
+		GError		*err = NULL;
 
-		err_clear(ri.min);
-		if(fut_cd(src->dir.path, old, sizeof old))
-		{
-			if(ri.page == 0)
-				err = rename_simple(&ri);
-			else if(ri.page == 1)
-				err = rename_regexp(&ri);
-			else if(ri.page == 2)
-				err = rename_map(&ri);
-			else
-				err = rename_case(&ri);
-			fut_cd(old, NULL, 0U);
-		}
+		if(ri.page == 0)
+			ok = rename_simple(&ri, &err);
+		else if(ri.page == 1)
+			ok = rename_regexp(&ri, &err);
+		else if(ri.page == 2)
+			ok = rename_map(&ri, &err);
 		else
-			err_set(ri.min, errno, "CD", src->dir.path);
-		if(!err && !errno)
+			ok = rename_case(&ri, &err);
+		if(ok)
 			dp_rescan_post_cmd(ri.src);
-		else
-			err_show(ri.min);
 	}
 	dp_free_selection(ri.selection);
 	ri.selection = NULL;
 
-	return 1;
+	return ok;
 }
