@@ -26,7 +26,6 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <regex.h>
 
 #include <gdk/gdkkeysyms.h>
 
@@ -44,22 +43,29 @@
 
 /* ----------------------------------------------------------------------------------------- */
 
-#define	SM_SET			(0)
-#define	SM_SET_ALL		(0)
-#define	SM_SET_SELECTED		(1)
-#define	SM_SET_UNSELECTED	(2)
 
-#define	SM_TYPE			(1)
-#define	SM_TYPE_ALL		(0)
-#define	SM_TYPE_DIRS		(1)
-#define	SM_TYPE_FILES		(2)
+/* Selection parameters.
+ * FIXME: Converted over from #defines, could do with some more loving.
+*/
+enum {
+	SM_SET = 0,
+	SM_TYPE = 1,
+	SM_ACTION = 2,
 
-#define	SM_ACTION		(2)
-#define	SM_ACTION_SELECT	(0)
-#define	SM_ACTION_UNSELECT	(1)
-#define	SM_ACTION_TOGGLE	(2)
+	SM_SET_ALL = 0,
+	SM_SET_SELECTED = 1,
+	SM_SET_UNSELECTED = 2,
 
-typedef void (*SelAction)(DirPane *, gint);
+	SM_TYPE_ALL = 0,
+	SM_TYPE_DIRS = 1,
+	SM_TYPE_FILES = 2,
+
+	SM_ACTION_SELECT = 0,
+	SM_ACTION_UNSELECT = 1,
+	SM_ACTION_TOGGLE = 2
+};
+
+typedef void (*SelAction)(DirPane *, const DirRow2 *);
 
 /* Base selection parameters. Filled-in either by GUI, or by direct command arguments. */
 typedef struct {
@@ -93,43 +99,53 @@ static SelParamShell old_sp_shell = {
 	NULL
 };
 				
-
 /* ----------------------------------------------------------------------------------------- */
 
 /* Select a specific row, or, if the row=ROW keyword is not specified, the one that was last clicked. */
 gint cmd_selectrow(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	GdkEventButton	*evt;
-	const gchar	*num;
-	gint		row = -1;
+	gint		ri;
+	gchar		buf[16];
+	GtkTreeIter	iter;
 
-	if((num = car_keyword_get_value(ca, "row", NULL)) != NULL)
+	ri = car_keyword_get_integer(ca, "row", -1);
+	if(ri < 0)
 	{
-		gchar	*endptr;
+		GdkEventButton	*evt;
+		gint		ok = 0;
 
-		row = strtol(num, &endptr, 0);
-		if(!*num || *endptr)
-			row = -1;
-	}
-	else if((evt = (GdkEventButton *) evt_event_get(GDK_BUTTON_PRESS)) != NULL)
-	{
-		gint	col;
+		if((evt = (GdkEventButton *) evt_event_get(GDK_BUTTON_PRESS)) != NULL)
+		{
+			GtkTreePath	*path;
 
-		if(!gtk_clist_get_selection_info(GTK_CLIST(src->list), evt->x, evt->y, &row, &col) || (row > src->dir.num_rows))
-			row = -1;
+			if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(src->view), evt->x, evt->y, &path, NULL, NULL, NULL))
+			{
+				GtkTreeIter	iter;
+
+				if(gtk_tree_model_get_iter(dp_get_tree_model(src), &iter, path))
+				{
+					dp_select(src, &iter);
+					ok = 1;
+				}
+				gtk_tree_path_free(path);
+			}
+		}
+		return ok;
 	}
 
-	if(row != -1)
+	g_snprintf(buf, sizeof buf, "%d", ri);
+	if(gtk_tree_model_get_iter_from_string(dp_get_tree_model(src), &iter, buf))
 	{
-		guint	action;
-		void	(*afunc)(DirPane *dp, gint row);
+		guint		action;
+		void		(*afunc)(DirPane *dp, const DirRow2 *row);
 
 		action = car_keyword_get_enum(ca, "action", 0, "select", "unselect", "toggle", NULL);
 		afunc = (action == 0) ? dp_select : (action == 1) ? dp_unselect : dp_toggle;
-		afunc(src, row);
+		afunc(src, (DirRow2 *) &iter);
 		gui_events_flush();
+		return 1;
 	}
-	return 1;
+	return 0;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -157,55 +173,26 @@ gint cmd_selectnone(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 /* 1998-06-04 -	Invert the selection set, making all currently selected files become
 **		unselected, and all unselected become selected.
 ** 2002-08-25 -	Complete rewrite, now uses bit vector for speed. 40-100X faster.
+** 2009-10-01 -	Complete rewrite, ported to GTK+ 2.0, one-third the size.
 */
 gint cmd_selecttoggle(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	guint	*bm, word, row, mask;
-	gsize	alloc;
-	GSList	*slist;
+	GtkTreeIter	iter;
 
-	/* Compute size of, and allocate selection bit vector. */
-	alloc = (src->dir.num_rows + CHAR_BIT * sizeof *bm - 1) / (CHAR_BIT * sizeof *bm);
-	bm = g_malloc(alloc * sizeof *bm);
-	memset(bm, 0, alloc * sizeof *bm);	/* Assume no selection. */
-
-	gtk_clist_freeze(src->list);
-
-	/* Was there a selection? If so, set bits for selected rows. */
-	if((slist = dp_get_selection(src)) != NULL)
+	if(gtk_tree_model_get_iter_first(dp_get_tree_model(src), &iter))
 	{
-		const GSList	*iter;
-
-		for(iter = slist; iter != NULL; iter = g_slist_next(iter))
+		GtkTreeSelection	*ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(src->view));
+		g_signal_handler_block(G_OBJECT(ts), src->sig_sel_changed);
+		do
 		{
-			register guint	n;
-
-			n = DP_ROW_INDEX(src, DP_SEL_ROW(iter));
-			bm[n / (CHAR_BIT * sizeof *bm)] |= 1 << (n % (CHAR_BIT * sizeof *bm));
-		}
-		dp_free_selection(slist);
+			if(gtk_tree_selection_iter_is_selected(ts, &iter))
+				gtk_tree_selection_unselect_iter(ts, &iter);
+			else
+				gtk_tree_selection_select_iter(ts, &iter);
+		} while(gtk_tree_model_iter_next(dp_get_tree_model(src), &iter));
+		g_signal_handler_unblock(G_OBJECT(ts), src->sig_sel_changed);
+		g_signal_emit_by_name(G_OBJECT(ts), "changed", src);
 	}
-	/* Now invert the selection bitmap. */
-	for(word = 0; word < alloc; word++)
-		bm[word] = ~bm[word];
-	/* Clear selection, and apply the (inverted) bitmap. */
-	dp_unselect_all(src);
-	for(word = row = 0; row < src->dir.num_rows; word++)
-	{
-		if(!bm[word])		/* Test a bunch (typically 32) rows in parallel. */
-		{
-			row += CHAR_BIT * sizeof *bm;
-			continue;
-		}
-		for(mask = 1; mask; mask <<= 1, row++)
-		{
-			if(bm[word] & mask)
-				dp_select(src, row);
-		}
-	}
-	gtk_clist_thaw(src->list);
-	g_free(bm);
-
 	return 1;
 }
 
@@ -214,24 +201,24 @@ gint cmd_selecttoggle(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *c
 /* 1999-05-06 -	Inspect which set (selected or unselected) the given row is, and determine
 **		if it complies with the wanted one as specified in <sp>.
 */
-static gboolean filter_set(DirPane *dp, guint row, SelParam *sp)
+static gboolean filter_set(DirPane *dp, const DirRow2 *row, const SelParam *sp)
 {
 	gboolean	sel;
 
 	if(sp->set == SM_SET_ALL)
 		return TRUE;
-	sel = dp_is_selected(dp, &dp->dir.row[row]);
+	sel = dp_is_selected(dp, row);
 	return (sp->set == SM_SET_SELECTED) ? sel : !sel;
 }
 
 /* 1999-05-06 -	Inspect type (dir or non-dir) given row has, and match against <sp>. */
-static gboolean filter_type(DirPane *dp, guint row, SelParam *sp)
+static gboolean filter_type(DirPane *dp, const DirRow2 *row, const SelParam *sp)
 {
 	gboolean	dir;
 
 	if(sp->type == SM_TYPE_ALL)
 		return TRUE;
-	dir = S_ISDIR(DP_ROW_LSTAT(&dp->dir.row[row]).st_mode);
+	dir = dp_row_get_file_type(dp_get_tree_model(dp), row, TRUE) == G_FILE_TYPE_DIRECTORY;
 
 	return (sp->type == SM_TYPE_DIRS) ? dir : !dir;
 }
@@ -259,12 +246,12 @@ static SelAction select_action_func(gint action)
 {
 	switch(action)
 	{
-		case SM_ACTION_SELECT:
-			return dp_select;
-		case SM_ACTION_UNSELECT:
-			return dp_unselect;
-		case SM_ACTION_TOGGLE:
-			return dp_toggle;
+	case SM_ACTION_SELECT:
+		return dp_select;
+	case SM_ACTION_UNSELECT:
+		return dp_unselect;
+	case SM_ACTION_TOGGLE:
+		return dp_toggle;
 	}
 	return NULL;
 }
@@ -275,15 +262,15 @@ static SelAction select_action_func(gint action)
 static void do_selectre(MainInfo *min, DirPane *dp, SelParamRE *sp)
 {
 	GString		*tmp;
-	regex_t		re;
-	void		(*sel_action)(DirPane *dp, gint row) = NULL;
+	GRegex		*re;
+	void		(*sel_action)(DirPane *dp, const DirRow2 *row) = NULL;
 
 	if(sp->re == NULL)
 		return;
 	sel_action = select_action_func(sp->basic.action);
 	if((tmp = g_string_new(sp->re->str)) != NULL)
 	{
-		gint	reret;
+		GError	*error = NULL;
 
 		if(sp->glob)
 			stu_gstring_glob_to_re(tmp);
@@ -292,33 +279,37 @@ static void do_selectre(MainInfo *min, DirPane *dp, SelParamRE *sp)
 			g_string_prepend_c(tmp, '^');
 			g_string_append_c(tmp,  '$');
 		}
-		if((reret = regcomp(&re, tmp->str, REG_EXTENDED | (sp->nocase ? REG_ICASE : 0))) == 0)
+		if((re = g_regex_new(tmp->str, G_REGEX_EXTENDED | (sp->nocase ? G_REGEX_CASELESS : 0), G_REGEX_MATCH_NOTEMPTY, &error)) != NULL)
 		{
-			gchar	*text;
-			gint	matchok;
-			guint	i, column;
+			gboolean	matchok;
+			GtkTreeIter	iter;
+			gchar		text[1024];
+			GtkTreeModel	*m = dp_get_tree_model(dp);
 
-			dp_freeze(dp);
-			matchok = sp->invert ? REG_NOMATCH : 0;
-			column  = compute_column(dp, sp->basic.content);
-			for(i = 0; i < dp->dir.num_rows; i++)
+			matchok = !sp->invert;
+			if(gtk_tree_model_get_iter_first(m, &iter))
 			{
-				if(filter_set(dp, i, &sp->basic) && filter_type(dp, i, &sp->basic))
+				do
 				{
-					gtk_clist_get_text(GTK_CLIST(dp->list), i, column, &text);
-					if(regexec(&re, text, 0, NULL, 0) == matchok)
-						sel_action(dp, i);
-				}
+					if(filter_set(dp, &iter, &sp->basic) && filter_type(dp, &iter, &sp->basic))
+					{
+						if(dpf_get_content(min, dp, &iter, sp->basic.content, text, sizeof text))
+						{
+							if(g_regex_match(re, text, G_REGEX_MATCH_NOTEMPTY, NULL) == matchok)
+								sel_action(dp, &iter);
+						}
+					}
+				} while(gtk_tree_model_iter_next(m, &iter));
 			}
-			dp_thaw(dp);
-			regfree(&re);
+			g_regex_unref(re);
 		}
 		else
 		{
-			gchar	ebuf[1024];
-
-			regerror(reret, &re, ebuf, sizeof ebuf);
-			dlg_dialog_async_new_error(ebuf);
+			if(error != NULL)
+			{
+				dlg_dialog_async_new_error(error->message);
+				g_error_free(error);
+			}
 		}
 		g_string_free(tmp, TRUE);
 	}
@@ -326,47 +317,25 @@ static void do_selectre(MainInfo *min, DirPane *dp, SelParamRE *sp)
 
 /* 1999-05-06 -	Build a mode selection menu, using items from <def>. Items are separated by vertical
 **		bars. Each item will have it's user data set to the index of the item in question.
+** 2010-05-15 -	Now builds a combo box, rather than a menu.
 */
-static GtkWidget * build_mode_menu(const gchar *def)
+static GtkWidget * build_mode_combo_box(const gchar *def)
 {
-	GtkWidget	*menu = NULL, *item;
+	GtkWidget	*cbox = NULL;
 	gchar		*temp, *iname, *iend;
 	guint		minor;
 
 	if((temp = g_strdup(def)) != NULL)
 	{
-		menu = gtk_menu_new();
+		cbox = gtk_combo_box_text_new();
 		for(iname = temp, minor = 0; (*iname != '\0') && (iend = strchr(iname, '|')) != NULL; iname = iend + 1, minor++)
 		{
 			*iend = '\0';
-			item = gtk_menu_item_new_with_label(iname);
-			gtk_menu_append(GTK_MENU(menu), item);
-			gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(minor));
+			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(cbox), iname);
 		}
 		g_free(temp);
 	}
-	return menu;
-}
-
-/* 1999-11-14 -	Build a menu with one choice per active content type in the given pane. */
-static GtkWidget * build_content_menu(DirPane *dp)
-{
-	MainInfo	*min = dp->main;
-	DPFormat	*dpf;
-	GtkWidget	*menu, *item;
-	guint		i;
-
-	dpf = &min->cfg.dp_format[dp->index];
-	menu = gtk_menu_new();
-	for(i = 0; i < dpf->num_columns; i++)
-	{
-		if(dpf->format[i].content == DPC_ICON)
-			continue;
-		item = gtk_menu_item_new_with_label(dpf_get_content_name(dpf->format[i].content));
-		gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(dpf->format[i].content));
-		gtk_menu_append(GTK_MENU(menu), item);
-	}
-	return menu;
+	return cbox;
 }
 
 /* 2005-06-04 -	Compute which index in a content menu (as created by build_content_menu(), above)
@@ -397,7 +366,7 @@ static void selparam_init_menus(DirPane *dp, GtkWidget *vbox, GtkWidget *opt_men
 				  N_("All types|Directories only|Non-directories only|"),
 				  N_("Select|Unselect|Toggle|") },
 			*mlabel[] = { N_("Set"), N_("Type"), N_("Action") };
-	GtkWidget	*table, *label, *menu;
+	GtkWidget	*table, *label;
 	guint		i;
 
 	table = gtk_table_new(3, 3, FALSE);
@@ -405,18 +374,14 @@ static void selparam_init_menus(DirPane *dp, GtkWidget *vbox, GtkWidget *opt_men
 	{
 		label = gtk_label_new(_(mlabel[i]));
 		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, i + 1, 0,0,0,0);
-		menu = build_mode_menu(_(mdef[i]));
-		opt_menu[i] = gtk_option_menu_new();
-		gtk_option_menu_set_menu(GTK_OPTION_MENU(opt_menu[i]), menu);
-		gtk_option_menu_set_history(GTK_OPTION_MENU(opt_menu[i]), *history[i]);
+		opt_menu[i] = build_mode_combo_box(_(mdef[i]));
+		gtk_combo_box_set_active(GTK_COMBO_BOX(opt_menu[i]), *history[i]);
 		gtk_table_attach(GTK_TABLE(table), opt_menu[i], 1, 2, i, i+1, GTK_EXPAND|GTK_FILL,0,0,0);
 	}
 	label = gtk_label_new(_("Content"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, i + 1, 0,0,0,0);
-	menu = build_content_menu(dp);
-	opt_menu[3] = gtk_option_menu_new();
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(opt_menu[3]), menu);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(opt_menu[3]), content_menu_find(dp, *history[3]));
+	opt_menu[3] = dpf_get_content_combo_box(NULL, NULL);	/* We read out choice later, no callback needed. */
+	gtk_combo_box_set_active(GTK_COMBO_BOX(opt_menu[3]), content_menu_find(dp, *history[3]));
 	gtk_table_attach(GTK_TABLE(table), opt_menu[3], 1, 2, i, i + 1, GTK_EXPAND|GTK_FILL,0,0,0);
 
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
@@ -451,17 +416,10 @@ static void selparam_get_value_pointers(guint **ptr, SelParam *sp)
 
 static void selparam_read_menus(guint **ptr, GtkWidget *opt_menu[])
 {
-	GtkWidget	*menu, *mi;
-	guint		i;
+	guint	i;
 
 	for(i = 0; i < 4; i++)
-	{
-		if((menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(opt_menu[i]))) != NULL)
-		{
-			if((mi = gtk_menu_get_active(GTK_MENU(menu))) != NULL)
-				*ptr[i] = GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(mi)));
-		}
-	}
+		*ptr[i] = gtk_combo_box_get_active(GTK_COMBO_BOX(opt_menu[i]));
 }
 
 /* 1998-06-04 -	Pop up a dialog window asking the user for a regular expression, and
@@ -525,7 +483,7 @@ gint cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 		if(sp.re != NULL)
 		{
 			gtk_entry_set_text(GTK_ENTRY(re), sp.re->str);
-			gtk_entry_select_region(GTK_ENTRY(re), 0, -1);
+			gtk_editable_select_region(GTK_EDITABLE(re), 0, -1);
 		}
 		gtk_box_pack_start(GTK_BOX(vbox), re, FALSE, FALSE, 0);
 		dlg = dlg_dialog_sync_new(vbox, _("Select Using RE"), NULL);
@@ -533,20 +491,11 @@ gint cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 		res = dlg_dialog_sync_wait(dlg);
 		if(res == DLG_POSITIVE)		/* If user accepted, collect widget state into 'sp'. */
 		{
-			GtkWidget	*menu, *mi;
-			guint		i;
+			guint	i;
 
 			for(i = 0; i < 3; i++)
-			{
-				if((menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(opt_menu[i]))) != NULL)
-				{
-					if((mi = gtk_menu_get_active(GTK_MENU(menu))) != NULL)
-						*mptr[i] = GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(mi)));
-				}
-			}
-			menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(opt_menu[3]));
-			mi   = gtk_menu_get_active(GTK_MENU(menu));
-			sp.basic.content = (DPContent) GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(mi)));
+				*mptr[i] = gtk_combo_box_get_active(GTK_COMBO_BOX(opt_menu[i]));
+			sp.basic.content = gtk_combo_box_get_active(GTK_COMBO_BOX(opt_menu[3]));
 			sp.glob	  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glob));
 			sp.invert = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(invert));
 			sp.full   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(full));
@@ -575,20 +524,25 @@ gint cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 */
 gint cmd_selectext(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	GSList	*slist, *iter;
-	guint	i;
+	GSList		*slist, *iter;
+	GtkTreeIter	riter;
+	GtkTreeModel	*model;
 
 	slist = dp_get_selection(src);
-
+	model = dp_get_tree_model(src);
 	for(iter = slist; iter != NULL; iter = g_slist_next(iter))
 	{
-		for(i = 0; i < src->dir.num_rows; i++)
-		{
-			if(DP_SEL_INDEX(src, iter) == i)
-				continue;
+		const DirRow2	*sel = iter->data;
 
-			if(stl_styleinfo_style_siblings(min->cfg.style, DP_SEL_TYPE(iter)->style, DP_ROW_TYPE(&src->dir.row[i])->style))
-				dp_select(src, i);
+		if(gtk_tree_model_get_iter_first(model, &riter))
+		{
+			do
+			{
+				if(memcmp(sel, &riter, sizeof *sel) == 0)
+					continue;
+				if(stl_styleinfo_style_siblings(min->cfg.style, dp_row_get_ftype(model, sel)->style, dp_row_get_ftype(model, &riter)->style))
+					dp_select(src, &riter);
+			} while(gtk_tree_model_iter_next(model, &riter));
 		}
 	}
 	dp_free_selection(slist);
@@ -604,40 +558,54 @@ gint cmd_selectext(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 */
 gint cmd_selectsuffix(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	GdkEventButton	*evt;
-	gint		row = -1;
 	const gchar	*suff = NULL;
+	GtkTreeModel	*m = dp_get_tree_model(src);
 
 	if((suff = car_bareword_get(ca, 0)) != NULL)
 		;
 	if(suff == NULL)
 	{
+		GdkEventButton	*evt;
+		GtkTreeIter	si;
+		gboolean	siok = FALSE;
+
 		if((evt = (GdkEventButton *) evt_event_get(GDK_BUTTON_PRESS)) != NULL)
 		{
-			gint	col;
+			GtkTreePath	*path = NULL;
 
-			if(!gtk_clist_get_selection_info(GTK_CLIST(src->list), evt->x, evt->y, &row, &col) || (row > src->dir.num_rows))
-				row = -1;
+			if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(src->view), evt->x, evt->y, &path, NULL, NULL, NULL) && path != NULL)
+			{
+				siok = gtk_tree_model_get_iter(m, &si, path);
+				gtk_tree_path_free(path);
+			}
 		}
 		else
-			row = src->focus_row;
-		if(row != -1)
-			suff = stu_strcasechr(DP_ROW_NAME(&src->dir.row[row]), '.');
+			g_warning("Suffix selection from focused row is not implemented");
+		if(siok)
+		{
+			const gchar	*dn = dp_row_get_name_display(dp_get_tree_model(src), (DirRow2 *) &si);
+			suff = g_utf8_strrchr(dn, -1, '.');
+		}
 	}
 
-	/* Did we find a row? If we did, extract its suffix, and select all matching rows. */
+	/* Did we find a suffix? If we did, extract its suffix, and select all matching rows. */
 	if(suff != NULL)
 	{
-		guint		action, i;
-		void		(*afunc)(DirPane *dp, gint row);
+		GtkTreeIter	iter;
 
-		action = car_keyword_get_enum(ca, "action", 0, "select", "unselect", "toggle", NULL);
-		afunc = (action == 0) ? dp_select : (action == 1) ? dp_unselect : dp_toggle;
-
-		for(i = 0; i < src->dir.num_rows; i++)
+		if(gtk_tree_model_get_iter_first(m, &iter))
 		{
-			if(stu_has_suffix(DP_ROW_NAME(&src->dir.row[i]), suff))
-				afunc(src, i);
+			guint	action;
+			void	(*afunc)(DirPane *dp, const DirRow2 *row);
+
+			action = car_keyword_get_enum(ca, "action", 0, "select", "unselect", "toggle", NULL);
+			afunc = (action == 0) ? dp_select : (action == 1) ? dp_unselect : dp_toggle;
+			do
+			{
+				if(stu_has_suffix(dp_row_get_name_display(m, &iter), suff))
+					afunc(src, &iter);
+
+			} while(gtk_tree_model_iter_next(m, &iter));
 		}
 	}
 	return 1;
@@ -650,41 +618,48 @@ gint cmd_selectsuffix(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *c
 */
 gint cmd_selecttype(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	GdkEventButton	*evt;
 	const gchar	*tname;
-	gint		row = -1;
 	GList		*types = NULL;
+	GtkTreeModel	*m = dp_get_tree_model(src);
+	GtkTreeIter	iter;
 
 	if((tname = car_bareword_get(ca, 0U)) != NULL)
 		types = typ_type_lookup_glob(min->cfg.type, tname);
 	else
 	{
-		if((evt = (GdkEventButton *) evt_event_get(GDK_BUTTON_PRESS)) != NULL)
-		{
-			gint	col;
+		GdkEventButton	*evt;
+		GtkTreePath	*path = NULL;
 
-			if(!gtk_clist_get_selection_info(GTK_CLIST(src->list), evt->x, evt->y, &row, &col) || (row > src->dir.num_rows))
-				row = -1;
-		}
+		if((evt = (GdkEventButton *) evt_event_get(GDK_BUTTON_PRESS)) != NULL)
+			gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(src->view), evt->x, evt->y, &path, NULL, NULL, NULL);
 		else
-			row = src->focus_row;
-		if(row != -1)
-			types = g_list_append(NULL, (gpointer) DP_ROW_TYPE(&src->dir.row[row]));
+			gtk_tree_view_get_cursor(GTK_TREE_VIEW(src->view), &path, NULL);
+
+		if(path != NULL)
+		{
+			GtkTreeIter	ei;
+
+			if(gtk_tree_model_get_iter(m, &ei, path))
+				types = g_list_append(NULL, dp_row_get_ftype(dp_get_tree_model(src), (DirRow2* ) &ei));
+			gtk_tree_path_free(path);
+		}
 	}
 
 	if(types != NULL)
 	{
-
-		guint	action, i;
-		void	(*afunc)(DirPane *dp, gint row);
-
-		action = car_keyword_get_enum(ca, "action", 0, "select", "unselect", "toggle", NULL);
-		afunc  = (action == 0) ? dp_select : (action == 1) ? dp_unselect : dp_toggle;
-
-		for(i = 0; i < src->dir.num_rows; i++)
+		if(gtk_tree_model_get_iter_first(m, &iter))
 		{
-			if(g_list_find(types, (gpointer) DP_ROW_TYPE(&src->dir.row[i])) != NULL)
-				afunc(src, i);
+			guint	action;
+			void	(*afunc)(DirPane *dp, const DirRow2 *row);
+
+			action = car_keyword_get_enum(ca, "action", 0, "select", "unselect", "toggle", NULL);
+			afunc  = (action == 0) ? dp_select : (action == 1) ? dp_unselect : dp_toggle;
+
+			do
+			{
+				if(g_list_find(types, dp_row_get_ftype(m, &iter)))
+					afunc(src, &iter);
+			} while(gtk_tree_model_iter_next(m, &iter));
 		}
 		g_list_free(types);
 	}
@@ -693,93 +668,28 @@ gint cmd_selecttype(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 /* ----------------------------------------------------------------------------------------- */
 
-#if 0
-struct cmp_rules {
-	gboolean	nocase;
-
-	gint		(*scmp)(const gchar *s1, const gchar *s2);
-};
-
-/* 2001-09-23 -	Compare <row> against all rows in <dst>, according to rules specified in <dr>.
-** 		Returns TRUE if there is a match, i.e. if a file named <row> exists in <dst>
-**		already.
-*/
-static gboolean cmp_exec(const DirRow *row, const DirPane *dst, const struct cmp_rules *dr)
-{
-	const gchar	*name;
-	gboolean	ret;
-	gint		i;
-	guint32		match;
-
-	name = DP_ROW_NAME(row);
-	for(i = 0; i < dst->dir.num_rows; i++)
-	{
-		if(dr->scmp(name, DP_ROW_NAME(&dst->dir.row[i])) != 0)
-			continue;
-		/* At this point, we have a name match. Furher requirements? */
-		return TRUE;
-	}
-	return FALSE;
-}
-
-gint cmd_selectcmp(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
-{
-	static struct cmp_rules	rules = { FALSE };
-	gint			i;
-
-	/* Select string comparison function to use. */
-	rules.scmp  = rules.nocase ? g_strcasecmp : strcmp;
-
-	for(i = 0; i < src->dir.num_rows; i++)
-	{
-		if(cmp_exec(&src->dir.row[i], dst, &rules))
-			dp_select(src, i);
-	}
-	return 1;
-}
-#endif
-/* ----------------------------------------------------------------------------------------- */
-
-typedef enum { SHELL_BOOLEAN } ShellMode;
+typedef enum {
+	SHELL_BOOLEAN = 0	/* Interpret shell exit status as boolean, 0 is success. */
+} ShellMode;
 
 typedef struct {
 	ShellMode	mode;
 } ShellRule;
 
-static int shell_select(char **argv, const char *dir, const ShellRule *rule)
+/* 2009-10-04 -	Rewritten, now uses glib primitive to run a full commandline, doesn't require splitting. */
+static gboolean shell_select(const gchar *commandline, const ShellRule *rule)
 {
-	pid_t	child;
+	gint	status;
 
-	if(argv == NULL || argv[0] == NULL || rule == NULL)
-		return 0;
-
-	if((child = fork()) >= 0)
+	if(g_spawn_command_line_sync(commandline, NULL, NULL, &status, NULL))
 	{
-		if(child == 0)	/* In child? */
+		switch(rule->mode)
 		{
-			fut_cd(dir, NULL, 0u);
-			execvp(argv[0], argv);
-			usleep(50000);
-			_exit(-1);
-		}
-		else if(child > 0)		/* In parent. */
-		{
-			int	stat = 0;
-
-			waitpid(child, &stat, 0);
-			if(WIFEXITED(stat))
-			{
-				stat = WEXITSTATUS(stat);
-				if(rule->mode == SHELL_BOOLEAN)
-					return stat == 0;
-				else
-					fprintf(stderr, __FILE__ ": Not handling shell select mode %d\n", rule->mode);
-			}
-			else
-				fprintf(stderr, __FILE__ ": Child command %s crashed, not interpreting return status\n", argv[0]);
+		case SHELL_BOOLEAN:
+			return WEXITSTATUS(status) == 0;
 		}
 	}
-	return 0;
+	return FALSE;
 }
 
 /* 2006-04-24 -	Select based on return value of shell command. */
@@ -788,8 +698,7 @@ gint cmd_selectshell(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca
 	SelParamShell	sp;
 	const gchar	*str;
 	GString		*cmd;
-	guint		i, column;
-	gchar		**argv;
+	guint		column;
 	gint		reply = DLG_POSITIVE;
 
 	/* Start from previous set of parameters. */
@@ -814,12 +723,13 @@ gint cmd_selectshell(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca
 		selparam_init_menus(src, vbox, opt_menu, mptr);
 		label = gtk_label_new(_("Enter shell command to run. The command\nwill have the selected content appended.\nAction is performed on successful exit."));
 		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-		entry = gtk_entry_new_with_max_length(1024);
+		entry = gtk_entry_new();
+		gtk_entry_set_max_length(GTK_ENTRY(entry), 1024);
 		if(sp.cmd != NULL)
 			gtk_entry_set_text(GTK_ENTRY(entry), sp.cmd->str);
 		gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
 		dlg = dlg_dialog_sync_new(vbox, _("Select using shell command"), _("OK|Cancel"));
-		gtk_entry_select_region(GTK_ENTRY(entry), 0, 1 << 30);
+		gtk_editable_select_region(GTK_EDITABLE(entry), 0, 1 << 30);
 		gtk_widget_grab_focus(entry);
 		reply = dlg_dialog_sync_wait(dlg);
 		if(reply == DLG_POSITIVE)
@@ -836,35 +746,32 @@ gint cmd_selectshell(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca
 	if(reply == DLG_POSITIVE)
 	{
 		SelAction	sel_action;
-		gchar		*text;
+		GtkTreeModel	*m = dp_get_tree_model(src);
+		GtkTreeIter	iter;
+		gchar		text[1024];
+		const ShellRule	rule = { SHELL_BOOLEAN };
 
-		column = compute_column(src, sp.basic.content);
-		cmd = g_string_sized_new(4096);		/* Static for simplicity. */
-		sel_action = select_action_func(sp.basic.action);
-		for(i = 0; i < src->dir.num_rows; i++)
+		if(gtk_tree_model_get_iter_first(m, &iter))
 		{
-			if(!filter_set(src, i, &sp.basic) || !filter_type(src, i, &sp.basic))
-				continue;
-			g_string_assign(cmd, sp.cmd->str);
-			g_string_append_c(cmd, ' ');
-			text = NULL;
-			gtk_clist_get_text(GTK_CLIST(src->list), i, column, &text);
-			if(text == NULL)
-				continue;
-			g_string_append(cmd, text);
-			if(cmd->str[0] == '\0' || cmd->str[0] == ' ')
-				continue;
-			argv = cpr_parse(min, cmd->str);
-			if(argv != NULL && argv[0] != NULL)
+			column = compute_column(src, sp.basic.content);
+			cmd = g_string_sized_new(4096);		/* Static for simplicity. */
+			sel_action = select_action_func(sp.basic.action);
+			do
 			{
-				const ShellRule	rule = { SHELL_BOOLEAN };
-
-				if(shell_select(argv, src->dir.path, &rule))
-					sel_action(src, i);
-				g_free(argv);
-			}
+				if(!filter_set(src, &iter, &sp.basic) || !filter_type(src, &iter, &sp.basic))
+					continue;
+				g_string_assign(cmd, sp.cmd->str);
+				g_string_append_c(cmd, ' ');
+				if(!dpf_get_content(min, src, &iter, sp.basic.content, text, sizeof text))
+					continue;
+				g_string_append(cmd, text);
+				if(cmd->str[0] == '\0' || cmd->str[0] == ' ')
+					continue;
+				if(shell_select(cmd->str, &rule))
+					sel_action(src, &iter);
+			} while(gtk_tree_model_iter_next(m, &iter));
+			g_string_free(cmd, TRUE);
 		}
-		g_string_free(cmd, TRUE);
 		old_sp_shell = sp;		/* Remember parameters for next time. */
 	}
 	return 0;
@@ -881,7 +788,7 @@ gint cmd_unselectfirst(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *
 
 	if((slist = dp_get_selection(src)) != NULL)
 	{
-		dp_unselect(src, DP_SEL_INDEX(src, slist));
+		dp_unselect(src, slist->data);
 		dp_free_selection(slist);
 	}
 	return 1;
