@@ -181,7 +181,7 @@ CmdSeq * csq_cmdseq_new(const gchar *name, guint32 flags)
 
 	if((cs = g_malloc(sizeof *cs)) != NULL)
 	{
-		stu_strncpy(cs->name, name, sizeof cs->name);
+		g_strlcpy(cs->name, name, sizeof cs->name);
 		cs->flags = flags;
 		cs->rows  = NULL;
 	}
@@ -197,7 +197,7 @@ CmdSeq * csq_cmdseq_copy(const CmdSeq *src)
 
 	if((dst = g_malloc(sizeof *dst)) != NULL)
 	{
-		stu_strncpy(dst->name, src->name, sizeof dst->name);
+		g_strlcpy(dst->name, src->name, sizeof dst->name);
 		dst->flags = src->flags;
 		dst->rows = NULL;
 		for(iter = src->rows; iter != NULL; iter = g_list_next(iter))
@@ -225,7 +225,7 @@ void csq_cmdseq_set_name(GHashTable *hash, CmdSeq *cs, const gchar *name)
 		g_hash_table_remove(hash, cs->name);
 		csq_cmdseq_filter_name(cs->name, name);
 		if(cs->name[0] == '\0' || g_hash_table_lookup(hash, cs->name) != NULL)
-			stu_strncpy(cs->name, csq_cmdseq_unique_name(hash), sizeof cs->name);
+			g_strlcpy(cs->name, csq_cmdseq_unique_name(hash), sizeof cs->name);
 		csq_cmdseq_hash(&hash, cs);
 	}
 }
@@ -390,11 +390,6 @@ gboolean csq_handle_ba_flags(MainInfo *min, guint32 flags)
 			return FALSE;
 	}
 
-	if(flags & CBAF_CD_SOURCE)
-		fut_cd(src->dir.path, NULL, 0);
-	else if(flags & CBAF_CD_DEST)
-		fut_cd(dst->dir.path, NULL, 0);
-
 	if(flags & CBAF_RESCAN_SOURCE)
 		dp_rescan_post_cmd(src);
 	if(flags & CBAF_RESCAN_DEST)
@@ -426,58 +421,68 @@ gboolean csq_handle_ba_flags(MainInfo *min, guint32 flags)
 */
 static gint fork_and_execute(MainInfo *min, CmdRow *row, gchar **argv)
 {
-	pid_t	child;
-	gint	estat, fd_out[2], fd_err[2];
-	CX_Ext	*ext = &row->extra.external;
-
-	if(!fut_locate_executable(g_getenv("PATH"), argv[0]))
-	{
-		err_set(min, errno, "Execute", argv[0]);
-		return 0;
-	}
+	GPid		child;
+	gchar		*working_directory = NULL;
+	gboolean	capture, ok;
+	gint		fd_out, fd_err, *fd_out_ptr = NULL, *fd_err_ptr = NULL;
+	CX_Ext		*ext = &row->extra.external;
+	GError		*err = NULL;
 
 	if(ext->gflags & CGF_KILLPREV)
 		chd_kill_child(argv[0]);
 
-	if(!cgr_grab_init(min, row, fd_out, fd_err))
-		return 0;
-
 	if(!csq_handle_ba_flags(min, ext->baflags[0]))
 		return 0;
 
-	if((child = fork()) >= 0)
+	/* Figure out where to CD, if requested and sensible. */
+	if(ext->baflags[0] & CBAF_CD_SOURCE)
 	{
-		if(child == 0)		/* In child process? */
-		{
-			if(cgr_set_child_pipes(min, row, fd_out, fd_err))
-			{
-				estat = execvp(argv[0], argv);
-			}
-			usleep(50000);				/* Someone, sue me. */
-			_exit(CHD_EXIT_FAILURE);
-		}
-		else			/* In parent. */
-		{
-			chd_register(argv[0], child, ext->gflags, ext->baflags[1]);
-			cgr_set_parent_pipes(min, row, argv[0], child, fd_out, fd_err);
-			kill(getpid(), SIGCHLD);		/* Resolves race condition? */
-		}
+		if(g_file_is_native(min->gui->cur_pane->dir.root))
+			working_directory = g_file_get_path(min->gui->cur_pane->dir.root);
 	}
-	err_show(min);
-	return errno == 0;
+	else if(ext->baflags[0] & CBAF_CD_DEST)
+	{
+		GFile	*droot = min->gui->pane[1 - min->gui->cur_pane->index].dir.root;
+		if(g_file_is_native(droot))
+			working_directory = g_file_get_path(droot);
+	}
+
+	capture = (row->type == CRTP_EXTERNAL && row->extra.external.gflags & CGF_GRABOUTPUT);
+	if(capture)
+	{
+		fd_out_ptr = &fd_out;
+		fd_err_ptr = &fd_err;
+	}
+	if((ok = g_spawn_async_with_pipes(working_directory, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, NULL, NULL, &child, NULL, fd_out_ptr, fd_err_ptr, &err)))
+	{
+		chd_register(argv[0], child, ext->gflags, ext->baflags[1]);
+		if(capture)
+			cgr_grab_output(min, argv[0], child, fd_out, fd_err);
+	}
+	if(working_directory != NULL)
+		g_free(working_directory);
+	if(!ok)
+	{
+		if(err)
+			err_set_gerror(min, &err, argv[0], NULL);
+		else
+			err_set(min, -1, argv[0], NULL);
+	}
+	return ok;
 }
 
 /* 1998-09-25 -	Execute an external command. */
 static gint execute_external(MainInfo *min, CmdRow *row)
 {
 	gchar	**argv;
+	gint	ok = 0;
 
 	if((argv = cpr_parse(min, row->def->str)) != NULL)
 	{
-		fork_and_execute(min, row, argv);
+		ok = fork_and_execute(min, row, argv);
 		cpr_free(argv);
 	}
-	return 1;
+	return ok;
 }
 
 /* 1998-09-25 -	Execute a single row, returning success or failure. */
@@ -553,13 +558,19 @@ static gint cmdseq_execute(MainInfo *min, CmdSeq *cs)
 gint csq_execute(MainInfo *min, const gchar *name)
 {
 	CmdSeq	*cs;
+	gint	ok;
 
 	if((name == NULL) || (*name == '\0'))
 		return 0;
 
 	if((min->cfg.commands.cmdseq != NULL) && ((cs = (CmdSeq *) g_hash_table_lookup(min->cfg.commands.cmdseq, name)) != NULL))
-		return cmdseq_execute(min, cs);
-	return execute_builtin(min, name);
+		ok = cmdseq_execute(min, cs);
+	else
+		ok = execute_builtin(min, name);
+	if(!ok)
+		err_show(min);
+
+	return ok;
 }
 
 /* 1999-06-06 -	Build a command from a printf()-style format string and suitable arguments, then
@@ -731,6 +742,7 @@ void csq_init_commands(MainInfo *min)
 	csq_add_builtin(min, "DpFocusPath",	cmd_dpfocuspath,	NULL);
 	csq_add_builtin(min, "DpFocusISrch",	cmd_dpfocusisrch,	cfg_dpfocusisrch);
 	csq_add_builtin(min, "DpGotoRow",	cmd_dpgotorow,		NULL);
+	csq_add_builtin(min, "DpMaximize",	cmd_dpmaximize,		NULL);
 
 	csq_add_builtin(min, "ActivateOther",	cmd_activateother,	NULL);
 	csq_add_builtin(min, "ActivateLeft",	cmd_activateleft,	NULL);
@@ -744,7 +756,7 @@ void csq_init_commands(MainInfo *min)
 	csq_add_builtin(min, "Move",		cmd_move,		NULL);
 	csq_add_builtin(min, "MoveAs",		cmd_moveas,		NULL);
 	csq_add_builtin(min, "Delete",		cmd_delete,		cfg_delete);
-	csq_add_builtin(min, "Rename",		cmd_rename,		NULL);
+	csq_add_builtin(min, "Rename",		cmd_rename,		cfg_rename);
 	csq_add_builtin(min, "RenameRE",	cmd_renamere,		NULL);
 	csq_add_builtin(min, "RenameSeq",	cmd_renameseq,		NULL);
 	csq_add_builtin(min, "ChMod",		cmd_chmod,		NULL);
@@ -781,6 +793,7 @@ void csq_init_commands(MainInfo *min)
 
 	csq_add_builtin(min, "About",		cmd_about,		NULL);
 	csq_add_builtin(min, "Run",		cmd_run,		NULL);
+	csq_add_builtin(min, "Rerun",		cmd_rerun,		NULL);
 	csq_add_builtin(min, "Configure",	cmd_configure,		cfg_configurecmd);
 	csq_add_builtin(min, "ConfigureSave",	cmd_configuresave,	NULL);
 	csq_add_builtin(min, "Quit", 		cmd_quit,		NULL);
