@@ -32,7 +32,6 @@
 #include "dirpane.h"
 #include "fileutil.h"
 #include "iconutil.h"
-#include "mount.h"
 #include "window.h"
 #include "xmlutil.h"
 
@@ -49,7 +48,7 @@
 #include "cfg_dirpane.h"
 #include "cfg_errors.h"
 #include "cfg_menus.h"
-#include "cfg_mount.h"
+#include "cfg_nag.h"
 #include "cfg_paths.h"
 #include "cfg_styles.h"
 #include "cfg_types.h"
@@ -82,11 +81,11 @@ typedef struct {
 static const CMDescribeFunc describe_page[] = {	cdp_describe,
 						ccs_describe, ccc_describe,
 						cst_describe, ctp_describe,
-/*						cmu_describe,*/
 						cbt_describe, cbl_describe,
-						cpt_describe, cmn_describe,
+						cpt_describe,
 						cwn_describe, cdl_describe,
-						cct_describe, cer_describe
+						cct_describe, cer_describe,
+						cng_describe,
 					};
 
 #define	CFG_PAGES	(sizeof describe_page / sizeof describe_page[0])
@@ -111,6 +110,8 @@ static void hide_config(CfgGui *cgu)
 	if(the_cfggui.flags & CFLG_RESET_KEYBOARD)
 		kbd_context_clear(cgu->min->gui->kbd_ctx);
 
+	if(the_cfggui.flags & CFLG_REBUILD_TOP)
+		rebuild_top(cgu->min);
 	if(the_cfggui.flags & CFLG_REBUILD_MIDDLE)
 		rebuild_middle(cgu->min);
 	if(the_cfggui.flags & CFLG_REBUILD_BOTTOM)
@@ -133,8 +134,6 @@ static void hide_config(CfgGui *cgu)
 	else if(the_cfggui.flags & CFLG_REDISP_RIGHT)
 		dp_redisplay_preserve(&cgu->min->gui->pane[1]);
 
-	if(the_cfggui.flags & CFLG_RESET_MOUNT)
-		mnt_init(cgu->min);
 	if(the_cfggui.flags & CFLG_RESET_KEYBOARD)
 		ctrl_keys_install(cgu->min->cfg.ctrlinfo, cgu->min->gui->kbd_ctx);
 
@@ -234,17 +233,23 @@ static void build_buttons(CfgGui *cgu)
 	cgu->ok = gtk_button_new_with_label(_("OK"));
 	g_signal_connect(G_OBJECT(cgu->ok), "clicked", G_CALLBACK(evt_ok_clicked), cgu);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->action_area), cgu->ok, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS(cgu->ok, GTK_CAN_DEFAULT);
-	gtk_widget_grab_default(cgu->ok);
-
 	cgu->save = gtk_button_new_with_label(_("Save"));
 	g_signal_connect(G_OBJECT(cgu->save), "clicked", G_CALLBACK(evt_save_clicked), cgu);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->action_area), cgu->save, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS(cgu->save, GTK_CAN_DEFAULT);
 	cgu->cancel = gtk_button_new_with_label(_("Cancel"));
 	g_signal_connect(G_OBJECT(cgu->cancel), "clicked", G_CALLBACK(evt_cancel_clicked), cgu);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->action_area), cgu->cancel, TRUE, TRUE, 0);
+
+#if GTK_CHECK_VERSION(2, 2, 0)
+	gtk_widget_set_can_default(cgu->ok, TRUE);
+	gtk_widget_set_can_default(cgu->save, TRUE);
+	gtk_widget_set_can_default(cgu->cancel, TRUE);
+#else
+	GTK_WIDGET_SET_FLAGS(cgu->ok, GTK_CAN_DEFAULT);
+	GTK_WIDGET_SET_FLAGS(cgu->save, GTK_CAN_DEFAULT);
 	GTK_WIDGET_SET_FLAGS(cgu->cancel, GTK_CAN_DEFAULT);
+#endif
+	gtk_widget_grab_default(cgu->ok);
 }
 
 /* 1998-06-16 -	This gets called as the user clicks the close button of the config GUI.
@@ -301,7 +306,7 @@ gint cfg_configure(MainInfo *min)
 				cfg_page[i]->update(min);
 		}
 		win_window_show(cgu->dlg);
-		gtk_notebook_set_page(GTK_NOTEBOOK(cgu->nbook), the_cfggui.page);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(cgu->nbook), the_cfggui.page);
 		gtk_widget_show_all(cgu->dlg);
 		gtk_grab_add(cgu->dlg);
 		return 1;
@@ -348,9 +353,12 @@ gint cfg_configure(MainInfo *min)
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 	g_signal_connect(G_OBJECT(view), "cursor_changed", G_CALLBACK(evt_view_cursor_changed), cgu);
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scwin), view);
-	gtk_box_pack_start(GTK_BOX(hbox), scwin, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), scwin, FALSE, FALSE, 0);
 
-	gtk_box_pack_start(GTK_BOX(hbox), cgu->nbook, TRUE, TRUE, 0);
+	scwin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scwin), cgu->nbook);
+	gtk_box_pack_start(GTK_BOX(hbox), scwin, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->vbox), hbox, TRUE, TRUE, 0);
 	build_buttons(cgu);
 
