@@ -105,8 +105,6 @@ CtrlInfo * ctrl_new_default(MainInfo *min)
 	ctrl_mouse_add(ctrl, 2, 0U, "DirParent");
 	ctrl_mouse_add(ctrl, 3, 0U, "MenuPopup");
 
-	ctrl_general_set_cmdseq(ctrl, CTRL_GENERAL_PATH_RMB, "MenuPopup menu=<ParentsMenu>");
-
 	return ctrl;
 }
 
@@ -222,7 +220,7 @@ static CtrlKey * key_new(const gchar *keyname, const gchar *cmdseq)
 	CtrlKey	*key;
 
 	key = g_malloc(sizeof *key);
-	stu_strncpy(key->keyname, keyname, sizeof key->keyname);
+	g_strlcpy(key->keyname, keyname, sizeof key->keyname);
 	if(cmdseq != NULL)
 		key->cmdseq = g_string_new(cmdseq);
 	else
@@ -339,7 +337,7 @@ void ctrl_key_set_keyname(CtrlInfo *ctrl, CtrlKey *key, const gchar *keyname)
 		if(g_hash_table_lookup(ctrl->keys, key->keyname) == key)
 		{
 			g_hash_table_remove(ctrl->keys, key->keyname);
-			stu_strncpy(key->keyname, keyname, sizeof key->keyname);
+			g_strlcpy(key->keyname, keyname, sizeof key->keyname);
 			g_hash_table_insert(ctrl->keys, key->keyname, key);
 		}
 	}
