@@ -17,6 +17,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "dialog.h"
+#include "window.h"
 
 struct Dialog {
 	GtkWidget	*dlg;		/* The GtkDialog widget used to display the dialog. */
@@ -101,8 +102,6 @@ static gboolean evt_sync_key_pressed(GtkWidget *wid, GdkEventKey *evt, gpointer
 {
 	Dialog	*dlg = user;
 
-	if(evt->keyval == GDK_Return)
-		gtk_dialog_response(GTK_DIALOG(dlg->dlg), 0);
 	if(evt->keyval == GDK_Escape)
 		gtk_dialog_response(GTK_DIALOG(dlg->dlg), dlg->last_button);
 	else
@@ -123,7 +122,7 @@ Dialog * dlg_dialog_sync_new(GtkWidget *body, const gchar *title, const gchar *b
 		buttons = _("_OK|_Cancel");
 
 	dlg = g_malloc(sizeof *dlg);
-	dlg->dlg = gtk_dialog_new();
+	dlg->dlg = win_dialog_open(GTK_WIDGET(the_state.main_window));
 	gtk_window_set_position(GTK_WINDOW(dlg->dlg), the_state.window_pos);
 	g_signal_connect(GTK_OBJECT(dlg->dlg), "key_press_event", G_CALLBACK(evt_sync_key_pressed), dlg);
 	g_signal_connect(GTK_OBJECT(dlg->dlg), "response", G_CALLBACK(evt_sync_response), dlg);
@@ -167,6 +166,16 @@ gint dlg_dialog_sync_new_simple_wait(const gchar *body, const gchar *title, cons
 	return ret;
 }
 
+/* 2009-12-29 -	Adds a widget to the dialog's action area. */
+void dlg_dialog_sync_action_add(Dialog *dlg, GtkWidget *wid)
+{
+	if(dlg != NULL && wid != NULL)
+	{
+		gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dlg->dlg)->action_area), wid, TRUE, TRUE, 0);
+		gtk_widget_show_all(wid);
+	}
+}
+
 /* 2002-08-20 -	Make the dialog stay open until destroyed. Used only by the generic command module,
 **		but on the other hand that module is kind of important.
 */
@@ -183,12 +192,9 @@ gint dlg_dialog_sync_wait(Dialog *dlg)
 {
 	if(dlg != NULL)
 	{
-		int	old_errno = errno;
-
 		gtk_widget_realize(dlg->dlg);
 		gtk_window_set_transient_for(GTK_WINDOW(dlg->dlg), the_state.main_window);
 		gtk_dialog_run(GTK_DIALOG(dlg->dlg));
-		errno = old_errno;
 		return dlg->button;
 	}
 	return -1;
@@ -247,7 +253,6 @@ static void evt_async_response(GtkWidget *wid, gint response, gpointer user)
 Dialog * dlg_dialog_async_new(GtkWidget *body, const gchar *title, const gchar *buttons, DlgAsyncFunc func, gpointer user)
 {
 	Dialog		*dlg;
-	gint		old_errno;
 
 	if(buttons == NULL)
 		buttons = _("_OK|_Cancel");
@@ -255,7 +260,7 @@ Dialog * dlg_dialog_async_new(GtkWidget *body, const gchar *title, const gchar *
 	dlg = g_malloc(sizeof *dlg);
 	dlg->func = func;
 	dlg->user = user;
-	dlg->dlg = gtk_dialog_new();
+	dlg->dlg = win_dialog_open(GTK_WIDGET(the_state.main_window));
 	gtk_window_set_position(&GTK_DIALOG(dlg->dlg)->window, the_state.window_pos);
 	g_signal_connect(G_OBJECT(dlg->dlg), "key_press_event", G_CALLBACK(evt_async_key_pressed), dlg);
 	g_signal_connect(G_OBJECT(dlg->dlg), "response", G_CALLBACK(evt_async_response), dlg);
@@ -265,10 +270,8 @@ Dialog * dlg_dialog_async_new(GtkWidget *body, const gchar *title, const gchar *
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->dlg)->vbox), body, TRUE, TRUE, 0);
 	dlg->last_button = add_buttons(dlg->dlg, buttons);
 	gtk_widget_realize(dlg->dlg);
-	old_errno = errno;
 	gtk_window_set_transient_for(GTK_WINDOW(dlg->dlg), the_state.main_window);
 	gtk_widget_show_all(dlg->dlg);
-	errno = old_errno;
 
 	return dlg;
 }
@@ -296,3 +299,30 @@ void dlg_dialog_async_close_silent(Dialog *dlg)
 {
 	async_close(dlg, -2);
 }
+
+/* ----------------------------------------------------------------------------------------- */
+
+static void evt_entry_changed(GtkWidget *entry, gpointer user)
+{
+	const gchar	*text = gtk_entry_get_text(GTK_ENTRY(entry));
+
+	dlg_dialog_set_positive_enabled(user, *text != '\0');
+}
+
+/* 2010-06-13 -	Make the dialog track the state of a GtkEntry, and only enable the "OK" button if entry is non-empty. */
+void dlg_dialog_track_entry(Dialog *dlg, GtkWidget *entry)
+{
+	if(dlg == NULL)
+		return;
+	g_signal_connect(G_OBJECT(entry), "changed", G_CALLBACK(evt_entry_changed), dlg);
+	/* Make sure it's synchronized initially, too. */
+	evt_entry_changed(entry, dlg);
+}
+
+/* 2010-09-05 -	Happy birthday, me! Enable/disable the "positive" (=OK) button on the fly. */
+void dlg_dialog_set_positive_enabled(Dialog *dlg, gboolean enabled)
+{
+	if(dlg == NULL)
+		return;
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(dlg->dlg), DLG_POSITIVE, enabled);
+}
