@@ -5,6 +5,8 @@
 #if !defined DIALOG_H
 #define	DIALOG_H
 
+#include "config.h"
+
 #include <glib.h>
 #include <gtk/gtk.h>
 
@@ -25,6 +27,7 @@ extern void	dlg_position_set(GtkWindowPosition pos);
 extern Dialog *	dlg_dialog_sync_new(GtkWidget *body, const gchar *title, const gchar *buttons);
 extern Dialog *	dlg_dialog_sync_new_simple(const gchar *body, const gchar *title, const gchar *buttons);
 extern gint	dlg_dialog_sync_new_simple_wait(const gchar *body, const gchar *title, const gchar *buttons);
+extern void	dlg_dialog_sync_action_add(Dialog *dlg, GtkWidget *wid);
 extern void	dlg_dialog_sync_stay_open(Dialog *dlg);
 extern gint	dlg_dialog_sync_wait(Dialog *dlg);
 extern void	dlg_dialog_sync_close(Dialog *dlg, gint button);
@@ -36,4 +39,7 @@ extern Dialog *	dlg_dialog_async_new_error(const gchar *body);
 extern void	dlg_dialog_async_close(Dialog *dlg, gint button);
 extern void	dlg_dialog_async_close_silent(Dialog *dlg);
 
+extern void	dlg_dialog_track_entry(Dialog *dlg, GtkWidget *entry);
+extern void	dlg_dialog_set_positive_enabled(Dialog *dlg, gboolean enabled);
+
 #endif		/* DIALOG_H */
