@@ -20,6 +20,7 @@
 **		and then creating a new (differently named) one. The filesystem can re-use the inode.
 ** 2008-01-26 -	Adapted to keep a separate UTF-8 copy of the path names, for GTK+ 2. Ugly, needs to
 **		be way better. Later? Heh.
+** 2010-02-09 -	Again, adapted to work with GIO. Assume less, use thicker gloves. Smooth.
 */
 
 #include "gentoo.h"
@@ -29,7 +30,6 @@
 #include <unistd.h>
 
 #include "cmdseq.h"
-#include "convutil.h"
 #include "dirpane.h"
 #include "errors.h"
 #include "fileutil.h"
@@ -44,40 +44,31 @@
 
 /* ----------------------------------------------------------------------------------------- */
 
-typedef struct {			/* A history key. A (device,inode) pair. */
-	dev_t	dev;
-	ino_t	inode;
-	guint	hash;
-} HKey;
-
 /* This is used to store the selected files in a pane, and nothing else. */
 struct DHSel {
-	GHashTable	*hash;		/* It's a hash of HKeys, as above. For fast 'apply' operations. */
+	GHashTable	*hash;		/* It's a hash of display file names, for fast apply(). */
+	GStringChunk	*chunk;		/* All keys in the hash are stored here. */
 };
 
 /* Info about a single remembered directory. The 'hist' field of dirpanes stores a list of these. */
 typedef struct {
-	gchar	dpath[2*PATH_MAX];	/* The path to this directory, in UTF-8. Must be first! FIXME: Kind of a gross assumption. */
-	gchar	path[PATH_MAX];		/* The path to this directory. */
-	gchar	realpath[PATH_MAX];	/* Canonical path, used as key since <dev,inode> isn't enough. */
-	DHSel	*sel;			/* The selection last time we were here. */
+	gchar	*parse_name;		/* The (VFs-friendly) "parse name" of the remembered directory. */
 	gfloat	vpos;			/* Vertical position. */
-	gint	focus_row;		/* Row that had the keyboard-controlled focus, or -1. */
+	DHSel	sel;			/* Selection, inlined but empty by default. */
 } DHEntry;
 
 struct DirHistory {
 	GList	*history;		/* Visited directories. Recent are close to head. */
 };
 
-/* Here's a glib GMemChunk we use to allocate HKeys, in the hope of getting better efficiency
-** than g_malloc() would give us. Note that there is just one such GMemChunk, from which we
-** allocate HKeys for _all_ directories (DHEntries).
-*/
-static GMemChunk	*the_chunk = NULL;
-
 /* ----------------------------------------------------------------------------------------- */
 
-static DHSel *	dirsel_set(DHSel *sel, const DirPane *dp);
+/* This is just a ballpark, for the average name of a GIO "parse name". Not critical. */
+static const gsize	NAME_LENGTH_AVERAGE = 64;
+
+static void	dirsel_new_in_place(DHSel *sel);
+static void	dirsel_set(DHSel *sel, const DirPane *dp);
+static void	dirsel_clear(DHSel *sel);
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -97,205 +88,135 @@ const gchar * dph_dirhistory_get_entry(const DirHistory *dh, guint index)
 {
 	if(dh != NULL)
 	{
-		const DHEntry   *dhe = (const DHEntry *) g_list_nth_data(dh->history, index);
+		const DHEntry   *dhe = g_list_nth_data(dh->history, index);
  		if(dhe != NULL)
-			return dhe->path;
+			return dhe->parse_name;
 	}
 	return NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-12-23 -	Set a history key according to information in <st>. */
-static void hkey_set(HKey *hk, const struct stat *st)
-{
-	if(hk != NULL && st != NULL)
-	{
-		hk->dev   = st->st_dev;
-		hk->inode = st->st_ino;
-	}
-}
-
-/* 1998-12-23 -	Create a new history key, initialized with information from <st>. */
-static HKey * hkey_new(struct stat *st)
-{
-	HKey	*hk = NULL;
-
-	if(the_chunk == NULL)
-		the_chunk = g_mem_chunk_new("HKey", sizeof *hk, 1024, G_ALLOC_AND_FREE);
-	if(the_chunk != NULL)
-	{
-		if((hk = g_mem_chunk_alloc(the_chunk)) != NULL)
-			hkey_set(hk, st);
-	}
-	return hk;
-}
-
-/* 1998-12-23 -	Compare two history keys for equality. */
-static gint hkey_equal(gconstpointer a, gconstpointer b)
-{
-	const HKey	*ha = a, *hb = b;
-
-	return (ha->dev == hb->dev) && (ha->inode == hb->inode);
-}
-
-/* 1998-12-23 -	Compute hash value from a history key. Nothing fancy. */
-static guint hkey_hash(gconstpointer a)
-{
-	const HKey	*ha = a;
-
-	return (guint) ha->dev ^ (guint) ha->inode;
-}
-
-/* 1998-12-23 -	A g_hash_table_foreach() callback that frees a hash key. Note that both <key>
-**		and <value> point at the same HKey structure here.
-*/
-static void hkey_free(gpointer key, gpointer value, gpointer user)
-{
-	g_mem_chunk_free(the_chunk, key);
-}
-
-/* 1999-06-06 -	A routine just like the one above, but for g_hash_table_foreach_remove(). */
-static gboolean hkey_free2(gpointer key, gpointer value, gpointer user)
-{
-	g_mem_chunk_free(the_chunk, key);
-
-	return TRUE;		/* Remove me, please. */
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1999-03-05 -	Create a new empty DHEntry structure. */
 static DHEntry * dhentry_new(void)
 {
 	DHEntry	*data;
 
 	data = g_malloc(sizeof *data);
-	data->dpath[0] = '\0';
-	data->path[0] = '\0';
-	data->realpath[0] = '\0';
-	data->sel = NULL;
-	data->vpos = 0.0f;
-	data->focus_row = -1;
+	data->parse_name = NULL;
+	data->vpos = 0.f;
+	/* Lazily assume there will be no actual selection to store. */
+	data->sel.hash = NULL;
+	data->sel.chunk = NULL;
 
 	return data;
 }
 
-static void dhentry_path_set(DHEntry *dhe, const gchar *path)
+static void dhentry_root_set(DHEntry *dhe, GFile *root)
 {
-	Conv	conv;
-	gchar	*dpl;
-
-	g_snprintf(dhe->path, sizeof dhe->path, "%s", path);
-	conv_init(&conv);
-	dpl = conv_add_reverse(&conv, path);
-	g_snprintf(dhe->dpath, sizeof dhe->dpath, "%s", dpl);
-	conv_end(&conv);
-}
-
-static void dhentry_realpath_set(DHEntry *dhe, const gchar *realpath)
-{
-	g_snprintf(dhe->realpath, sizeof dhe->realpath, "%s", realpath);
+	if(dhe == NULL || root == NULL)
+		return;
+	if(dhe->parse_name != NULL)
+		g_free(dhe->parse_name);
+	dhe->parse_name = g_file_get_parse_name(root);
 }
 
 /* 1999-06-09 -	Update <dhe> to mimic state of <dp>. */
 static void dhentry_update(DHEntry *dhe, const DirPane *dp)
 {
 	if(dp->main->cfg.dp_history.select)
-		dhe->sel = dirsel_set(dhe->sel, dp);
-	else
 	{
-		dph_dirsel_destroy(dhe->sel);
-		dhe->sel = NULL;
+		if(dhe->sel.hash == NULL)
+			dirsel_new_in_place(&dhe->sel);
+		dirsel_set(&dhe->sel, dp);
 	}
-	dhe->vpos = dph_vpos_get(dp);
-	if(dp->focus_row != -1)
-		dhe->focus_row = dp->focus_row;
 	else
-		dhe->focus_row = -1;
+		dirsel_clear(&dhe->sel);
+	dhe->vpos = dph_vpos_get(dp);
 }
 
 /* 1999-06-09 -	Apply state conserved in <dhe> to the rows of <dp>. */
 static void dhentry_apply(const DHEntry *dhe, DirPane *dp)
 {
-	dph_dirsel_apply(dp, dhe->sel);
+	dph_dirsel_apply(dp, &dhe->sel);
 	dph_vpos_set(dp, dhe->vpos);
-	if(dhe->focus_row != -1)	/* Was focus active? */
-		dp_focus(dp, dhe->focus_row);
 }
 
 /* 1999-06-06 -	Destroy a directory history entry. */
 static void dhentry_destroy(DHEntry *dh)
 {
-	if(dh->sel != NULL)
-		g_hash_table_foreach_remove(dh->sel->hash, hkey_free2, NULL);
-	g_free(dh->sel);
+	if(dh->sel.hash != NULL)
+	{
+		g_hash_table_destroy(dh->sel.hash);
+		g_string_chunk_free(dh->sel.chunk);
+	}
 	g_free(dh);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-03-05 -	Create a new empty selection. */
-static DHSel * dirsel_new(void)
+static void dirsel_new_in_place(DHSel *sel)
 {
-	DHSel	*sel;
+	if(sel == NULL)
+		return;
 
-	sel = g_malloc(sizeof *sel);
-	sel->hash = g_hash_table_new(hkey_hash, hkey_equal);
-
-	return sel;
-}
-
-static gint dummy_func(gpointer a, gpointer b, gpointer c)
-{
-	return TRUE;
+	sel->hash = g_hash_table_new(g_str_hash, g_str_equal);
+	sel->chunk = g_string_chunk_new(NAME_LENGTH_AVERAGE);
 }
 
 static void dirsel_clear(DHSel *sel)
 {
 	if((sel == NULL) || (sel->hash == NULL))
 		return;
-
-	g_hash_table_foreach_remove(sel->hash, dummy_func, NULL);
+	g_hash_table_remove_all(sel->hash);
+	g_string_chunk_clear(sel->chunk);
 }
 
-/* 1999-03-05 -	Given an <sel> structure, replace selection with that of <dp>. Returns the
-**		new selection (or NULL if <dp> had no selected items). If <sel> is NULL on
-**		entry, a new selection will be created and returned.
+/* 1999-03-05 -	Given an <sel> structure, replace selection with that of <dp>.
+** 2010-02-09 -	Significant edits for GIO-adapted data structures.
 */
-static DHSel * dirsel_set(DHSel *sel, const DirPane *dp)
+static void dirsel_set(DHSel *sel, const DirPane *dp)
 {
 	GSList	*slist;
 
+	/* Always clear; if there is no selection clearing is what we want. */
+	dirsel_clear(sel);
 	if((slist = dp_get_selection_full(dp)) != NULL)
 	{
-		HKey		*key;
 		const GSList	*iter;
+		GtkTreeModel	*model = dp_get_tree_model(dp);
+		gsize		count = 0, length = 0;
 
-		if(sel == NULL)
-			sel = dirsel_new();
-		else
-			dirsel_clear(sel);
-		
 		for(iter = slist; iter != NULL; iter = g_slist_next(iter))
 		{
-			key = hkey_new(&DP_SEL_LSTAT(iter));
+			const gchar	*name = dp_row_get_name_display(model, iter->data);
+			gchar		*key;
+
+			key = g_string_chunk_insert(sel->chunk, name);
 			g_hash_table_insert(sel->hash, key, key);	/* *Value* is returned on lookup!! */
+			length += strlen(key);
+			count++;
 		}
 		dp_free_selection(slist);
-		return sel;
 	}
-	return NULL;
 }
 
 /* 1999-03-05 -	This returns an opaque representation of all selected rows of <dp>. The
 **		selection is not related to the order in which these rows are displayed
 **		in the pane, so it's handy to use before e.g. resorting the pane.
-**		Note that NULL is a valid representation if there is no selection.
+** 2010-02-09 -	Changed around a bit to match the new definition of DHSel. No longer OK
+**		to return NULL to represent "no selection".
 */
 DHSel * dph_dirsel_new(DirPane *dp)
 {
-	return dirsel_set(NULL, dp);
+	DHSel	*sel;
+
+	sel = g_malloc(sizeof *sel);
+	dirsel_new_in_place(sel);
+	dirsel_set(sel, dp);
+
+	return sel;
 }
 
 /* 1999-03-05 -	Apply given given <sel> selection to <dp>, making those rows selected
@@ -305,17 +226,23 @@ DHSel * dph_dirsel_new(DirPane *dp)
 */
 void dph_dirsel_apply(DirPane *dp, const DHSel *sel)
 {
-	HKey	key;
-	guint	i;
+	GtkTreeModel	*model;
+	DirRow2		iter;
 
 	if((sel == NULL) || (sel->hash == NULL))
 		return;
 
-	for(i = 0; i < dp->dir.num_rows; i++)
+	model = dp_get_tree_model(dp);
+	if(gtk_tree_model_get_iter_first(model, &iter))
 	{
-		hkey_set(&key, &DP_ROW_LSTAT(&dp->dir.row[i]));
-		if(g_hash_table_lookup(sel->hash, &key))
-			dp_select(dp, i);
+		do
+		{
+			const gchar	*name;
+
+			name = dp_row_get_name(model, &iter);
+			if(g_hash_table_lookup(sel->hash, name) != NULL)
+				dp_select(dp, &iter);
+		} while(gtk_tree_model_iter_next(model, &iter));
 	}
 }
 
@@ -329,8 +256,8 @@ void dph_dirsel_destroy(DHSel *sel)
 
 	if(sel->hash != NULL)
 	{
-		g_hash_table_foreach(sel->hash, hkey_free, NULL);
 		g_hash_table_destroy(sel->hash);
+		g_string_chunk_free(sel->chunk);
 	}
 	g_free(sel);
 }
@@ -368,19 +295,22 @@ void dph_vpos_set(DirPane *dp, gfloat vpos)
 */
 void dph_state_save(DirPane *dp)
 {
-	gchar	rp[PATH_MAX];
+	gchar	*name;
 
-	if(dp->dir.num_rows == 0)		/* No directory shown? Then give up. */
+	if(dp->dir.root == NULL)
 		return;
-	if(realpath(dp->dir.path, rp) == NULL)
+
+	name = g_file_get_parse_name(dp->dir.root);
+	if(name == NULL)
 		return;
-	if((dp->hist->history == NULL) || strcmp(rp, ((DHEntry *) dp->hist->history->data)->realpath))
+
+	if((dp->hist->history == NULL) || strcmp(name, ((DHEntry *) dp->hist->history->data)->parse_name))
 	{
 		dp->hist->history = g_list_prepend(dp->hist->history, dhentry_new());
-		dhentry_path_set(dp->hist->history->data, dp->dir.path);
-		dhentry_realpath_set(dp->hist->history->data, rp);
+		dhentry_root_set(dp->hist->history->data, dp->dir.root);
 	}
 	dhentry_update(dp->hist->history->data, dp);
+	g_free(name);
 }
 
 /* 2002-06-13 -	Other half of the third-generation history interface. This restores any saved
@@ -389,18 +319,20 @@ void dph_state_save(DirPane *dp)
 void dph_state_restore(DirPane *dp)
 {
 	GList	*iter;
-	gchar	rp[PATH_MAX];
+	gchar	*name;
 	DHEntry	*entry;
 
 	/* Search for key matching current dir in history. */
-	if(realpath(dp->dir.path, rp) == NULL)
+	if((name = g_file_get_parse_name(dp->dir.root)) == NULL)
 		return;
 	for(iter = dp->hist->history, entry = NULL; iter != NULL; iter = g_list_next(iter))
 	{
 		entry = iter->data;
-		if(strcmp(rp, entry->realpath) == 0)
+		if(strcmp(name, entry->parse_name) == 0)
 			break;
 	}
+	g_free(name);
+
 	if(iter != NULL)
 	{
 		dhentry_apply(entry, dp);
@@ -414,8 +346,7 @@ void dph_state_restore(DirPane *dp)
 	else
 	{
 		dp->hist->history = g_list_prepend(dp->hist->history, dhentry_new());
-		dhentry_path_set(dp->hist->history->data, dp->dir.path);
-		dhentry_realpath_set(dp->hist->history->data, rp);
+		dhentry_root_set(dp->hist->history->data, dp->dir.root);
 		dhentry_apply(dp->hist->history->data, dp);
 		/* Prune history list, if it has grown too long. */
 		if(g_list_length(dp->hist->history) > HISTORY_SIZE)
@@ -432,12 +363,24 @@ void dph_state_restore(DirPane *dp)
 			}
 		}
 	}
-	/* Update path history widgets. Relies on <dpath> member being first in DHEntry. */
+	/* Update path history widgets. Relies on <parse_name> member being first in DHEntry. */
 	dp_history_set(dp, dp->hist->history);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2010-02-09 -	Gets the parse name from an indirect pointer, such as those handed (in a
+**		list) to dp_history_set(), above.
+*/
+const gchar * dph_entry_get_parse_name(const void *entry)
+{
+	const DHEntry *	e = entry;
+
+	if(e == NULL)
+		return NULL;
+	return e->parse_name;
+}
+
 /* 2002-02-15 -	Return first path from history for <dp>. */
 const gchar * dph_history_get_first(const DirPane *dp)
 {
@@ -479,9 +422,11 @@ void dph_history_save(MainInfo *min, const DirPane *dp, gsize num)
 		xml_put_node_open(out, "History");
 		for(iter = dp[i].hist->history; iter != NULL; iter = g_list_next(iter))
 		{
-			if(*(gchar *) iter->data == '\0')
+			const DHEntry	*entry = iter->data;
+
+			if(entry->parse_name[0] == '\0')
 				continue;
-			xml_put_text(out, "dir", iter->data);
+			xml_put_text(out, "root", entry->parse_name);
 		}
 		xml_put_node_close(out, "History");
 		xml_put_node_close(out, "Pane");
@@ -500,24 +445,13 @@ static void load_dir(const XmlNode *node, gpointer user)
 	const gchar	*text;
 	DHEntry		*ent;
 
-	if(!xml_get_text(node, "dir", &text))
+	if(!xml_get_text(node, "root", &text))
 		return;
 	if(text && *text == '\0')
 		return;
 	ent = dhentry_new();
-	stu_strncpy(ent->path, text, sizeof ent->path);
-	if(realpath(ent->path, ent->realpath) != NULL)
-	{
-		Conv		conv;
-		const gchar	*display;
-
-		display = conv_begin_reverse(&conv, ent->path);
-		g_strlcpy(ent->dpath, display, sizeof ent->dpath);
-		dp->hist->history = g_list_append(dp->hist->history, ent);
-		conv_end(&conv);
-	}
-	else
-		dhentry_destroy(ent);
+	ent->parse_name = g_strdup(text);
+	dp->hist->history = g_list_append(dp->hist->history, ent);
 }
 
 static void load_pane(const XmlNode *node, gpointer user)
