@@ -19,6 +19,7 @@
 #include "file.h"
 #include "fileutil.h"
 #include "gfam.h"
+#include "guiutil.h"
 #include "xmlutil.h"
 #include "strutil.h"
 #include "types.h"
@@ -30,18 +31,17 @@
 #include "cmdseq.h"
 #include "children.h"
 #include "dpformat.h"
-#include "mount.h"
 #include "keyboard.h"
 #include "controls.h"
 #include "iconutil.h"
 #include "cmdseq_config.h"
 #include "menus.h"
+#include "nag_dialog.h"
 
 #include "cfg_module.h"
+#include "cfg_nag.h"
 #include "cfg_windows.h"
 
-#include "graphics/icon_iconify.xpm"
-
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-05-18 -	Filter out files the user really doesn't want to see, and that recursive
@@ -56,16 +56,6 @@ static gboolean dir_filter(const gchar *name)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-10-11 -	This is called as GTK+ quits. Very handy. */
-static gint quit_handler(gpointer user)
-{
-	chd_kill_children();
-
-	return TRUE;
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 2000-03-18 -	User clicked close button on window. Just react as if the Quit command was run. */
 static gboolean evt_main_delete(GtkWidget *wid, GdkEventAny *evt, gpointer user)
 {
@@ -79,12 +69,6 @@ static void init_pane(DirPane *dp, gint index)
 {
 	dp->index = index;
 	dp->complete.prefix[0] = '\0';
-	dp->dir.row = NULL;
-	dp->dir.row_alloc = 0;
-	dp->dir.num_rows  = 0;
-	dp->dir.name = NULL;
-	dp->dir.name_alloc = 0;
-	dp->dir.name_use   = 0;
 	dp->dir.stat = NULL;
 	dp->dir.stat_alloc = 0;
 	dp->dir.stat_use   = 0;
@@ -95,27 +79,28 @@ static void init_pane(DirPane *dp, gint index)
 static void evt_paned_notify_position(GObject *obj, GParamSpec *spec, gpointer user)
 {
 	MainInfo	*min = user;
-	gint		pos, ww;
+	gint		pos, pw, ph, np;
 	gdouble		old;
 
-	pos = gtk_paned_get_position(GTK_PANED(obj));
-	gdk_window_get_size(min->gui->window->window, &ww, NULL);
-	/* If window is not configured yet, the size returned here is dubious. */
-	if(!min->gui->win_configured)
+	if(!dp_realized(min))
 		return;
+
+	pos = gtk_paned_get_position(GTK_PANED(obj));
+	gdk_window_get_size(min->gui->panes->window, &pw, &ph);
+	np = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? pw : ph;
 	old = min->cfg.dp_paning.value;
 	switch(min->cfg.dp_paning.mode)
 	{
 		case DPSPLIT_FREE:
 			break;
 		case DPSPLIT_RATIO:
-			min->cfg.dp_paning.value = (gdouble) pos / ww;
+			min->cfg.dp_paning.value = (gdouble) pos / np;
 			break;
 		case DPSPLIT_ABS_LEFT:
 			min->cfg.dp_paning.value = pos;
 			break;
 		case DPSPLIT_ABS_RIGHT:
-			min->cfg.dp_paning.value = ww - pos;
+			min->cfg.dp_paning.value = np - pos;
 			break;
 	}
 }
@@ -141,26 +126,46 @@ static GtkWidget * build_dirpanes(MainInfo *min)
 	gui->sig_pane_notify = g_signal_connect(G_OBJECT(gui->panes), "notify::position", G_CALLBACK(evt_paned_notify_position), min);
 	gtk_widget_show(gui->panes);
 
+	gtk_widget_grab_focus(min->gui->pane[0].view);
+
 	return gui->panes;
 }
 
+static void evt_top_button_press_event(GtkWidget *top, GdkEventButton *evt, gpointer user)
+{
+	if(evt->button == 1 && evt->type == GDK_2BUTTON_PRESS)
+		csq_execute(user, "About");
+}
+
 /* 1998-05-19 -	Create the top widgetry, returning something the caller can just add to a box. Sets the
 **		<label> pointer to a GtkLabel which can be used later to display funny messages and stuff.
 **		The label is internally wrapped in a GtkEventbox, which makes updating it possible without
 **		causing instant epilepsy in everyone watching. GTK+ really is smooth.
 ** 1998-11-26 -	Made the label focusable (?), in order to *finally* have somewhere safe to put the focus
 **		when I don't want it on path entry widgets. Nice.
+** 2010-10-03 -	Added sneaky click-listener, so that we can run About on double-click.
 */
-static GtkWidget * build_top(GtkWidget **label)
+static GtkWidget * build_top(MainInfo *min)
 {
+	GtkLabel	**label = (GtkLabel **) &min->gui->top;
 	GtkWidget	*top, *lab;
 
+	if(min->cfg.errors.display == ERR_DISPLAY_TITLEBAR)
+	{
+		if(*label != NULL)
+			gtk_widget_destroy(GTK_WIDGET(*label));
+		*label = NULL;
+		return NULL;
+	}
+
 	top = gtk_event_box_new();
 	lab = gtk_label_new("");
 	gtk_container_add(GTK_CONTAINER(top), lab);
 	gtk_widget_show_all(top);
 	if(label != NULL)
-		*label = lab;
+		*label = GTK_LABEL(lab);
+	g_signal_connect(G_OBJECT(top), "button_press_event", G_CALLBACK(evt_top_button_press_event), min);
+
 	return top;
 }
 
@@ -179,6 +184,25 @@ static GtkWidget * build_bottom(MainInfo *min)
 	return hbox;
 }
 
+void rebuild_top(MainInfo *min)
+{
+	GtkWidget	*top;
+
+	if(min->gui->top != NULL)
+	{
+		gtk_widget_destroy(min->gui->top);
+		min->gui->top = NULL;
+	}
+	top = build_top(min);
+	if(top != NULL)
+	{
+		gtk_box_pack_start(GTK_BOX(min->gui->vbox), top, FALSE, FALSE, 0);
+		gtk_box_reorder_child(GTK_BOX(min->gui->vbox), top, 0);
+	}
+	dp_show_stats(min->gui->cur_pane);
+	gui_set_main_title(min, NULL);
+}
+
 /* 1998-10-26 -	Rebuild the middle part of the GUI, i.e. the panes. */
 void rebuild_middle(MainInfo *min)
 {
@@ -196,6 +220,7 @@ void rebuild_middle(MainInfo *min)
 		gtk_paned_add2(GTK_PANED(min->gui->panes), right);
 	min->gui->sig_pane_notify = g_signal_connect(G_OBJECT(min->gui->panes), "notify::position", G_CALLBACK(evt_paned_notify_position), min);
 	gtk_widget_show(min->gui->panes);
+	dp_split_refresh(min);
 }
 
 /* 1998-07-14 -	Rebuild the bottom part of the GUI, whose main responsibility is to contain
@@ -209,66 +234,10 @@ void rebuild_bottom(MainInfo *min)
 	gtk_widget_show(min->gui->bottom);
 }
 
-/* 1999-03-26 -	Set icon for when gentoo gets iconified. Neato.
-** 2002-08-10 -	It seems gdk_window_set_icon_name() has problems when the locale gets set to
-**		something more interesting than "C". So we work around that.
-*/
-static void window_set_icon(GtkWidget *win)
-{
-	GdkPixmap	*pmap;
-	GdkBitmap	*bmap;
-
-	if((pmap = gdk_pixmap_create_from_xpm_d(win->window, &bmap, NULL, icon_iconify_xpm)) != NULL)
-	{
-		const gchar	*op;
-		gchar		old[256];
-
-		gdk_window_set_icon(win->window, NULL, pmap, bmap);
-#if defined ENABLE_NLS
-		if((op = setlocale(LC_ALL, NULL)) != NULL)
-		{
-			g_snprintf(old, sizeof old, "%s", op);
-			setlocale(LC_ALL, "C");
-		}
-#endif
-		gdk_window_set_icon_name(win->window, PACKAGE);
-#if defined ENABLE_NLS
-		if(op != NULL)
-			setlocale(LC_ALL, old);
-#endif
-		gdk_window_set_group(win->window, win->window);
-	}
-}
-
 /* 2008-04-20 -	Replaced old size_allocation-tracking code with new, using configure events instead. */
 static gboolean evt_main_configure(GtkWidget *wid, GdkEventConfigure *req, gpointer user)
 {
-	MainInfo	*min = user;
-	gint		pos = -1;
-
-	switch(min->cfg.dp_paning.mode)
-	{
-		case DPSPLIT_FREE:
-			return FALSE;
-		case DPSPLIT_RATIO:
-			pos = req->width * min->cfg.dp_paning.value;
-			break;
-		case DPSPLIT_ABS_LEFT:
-			pos = min->cfg.dp_paning.value;
-			break;
-		case DPSPLIT_ABS_RIGHT:
-			pos = req->width - min->cfg.dp_paning.value;
-			break;
-	}
-	if(pos > 0)
-	{
-		g_signal_handler_block(G_OBJECT(min->gui->window), min->gui->sig_main_configure);
-		g_signal_handler_block(G_OBJECT(min->gui->panes), min->gui->sig_pane_notify);
-		gtk_paned_set_position(GTK_PANED(min->gui->panes), pos);
-		g_signal_handler_unblock(G_OBJECT(min->gui->panes), min->gui->sig_pane_notify);
-		g_signal_handler_unblock(G_OBJECT(min->gui->window), min->gui->sig_main_configure);
-	}
-	min->gui->win_configured = TRUE;
+	dp_split_refresh(user);
 
 	return FALSE;	/* Keep propagating. */
 }
@@ -280,28 +249,25 @@ static GtkWidget * build_gui(MainInfo *min)
 	min->gui = g_malloc(sizeof *min->gui);
 	min->gui->window = NULL;
 	min->gui->window = win_window_open(min->cfg.wininfo, WIN_MAIN);
-	gtk_object_set_user_data(GTK_OBJECT(min->gui->window), min);
+	g_object_set_data(G_OBJECT(min->gui->window), "user", min);
 	min->gui->sig_main_configure = g_signal_connect(G_OBJECT(min->gui->window), "configure_event", G_CALLBACK(evt_main_configure), min);
 	min->gui->sig_main_delete = g_signal_connect(G_OBJECT(min->gui->window), "delete_event", G_CALLBACK(evt_main_delete), min);
-	min->gui->win_configured = FALSE;
 	min->gui->pane[0].main = min->gui->pane[1].main = min;
-	min->gui->pane[0].list = min->gui->pane[1].list = NULL;
 	min->gui->vbox = gtk_vbox_new(FALSE, 0);
-	min->gui->main_tt = gtk_tooltips_new();
 	min->gui->kbd_ctx = kbd_context_new(min);
 
-	top = build_top(&min->gui->top);
+	top = build_top(min);
 	if((panes = build_dirpanes(min)) != NULL)
 	{
 		bottom = build_bottom(min);
-		gtk_box_pack_start(GTK_BOX(min->gui->vbox), top,    FALSE, FALSE, 0);
+		if(top != NULL)
+			gtk_box_pack_start(GTK_BOX(min->gui->vbox), top,    FALSE, FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(min->gui->vbox), panes,  TRUE,  TRUE,  0);
 		gtk_box_pack_start(GTK_BOX(min->gui->vbox), bottom, FALSE, FALSE, 0);
 		gtk_widget_show_all(min->gui->vbox);
 		min->gui->middle = panes;
 		min->gui->bottom = bottom;
 		min->gui->cur_pane = &min->gui->pane[0];
-		gtk_tooltips_enable(min->gui->main_tt);
 		kbd_context_attach(min->gui->kbd_ctx, GTK_WINDOW(min->gui->window));
 		ctrl_keys_install(min->cfg.ctrlinfo, min->gui->kbd_ctx);
 		return min->gui->vbox;
@@ -348,7 +314,7 @@ static void load_gtk_rc(MainInfo *min)
 static void init_cmd_options(CfgInfo *cfg)
 {
 	cfg->opt_overwrite.show_info = TRUE;
-	stu_strncpy(cfg->opt_overwrite.datefmt, "%Y-%m-%d %H:%M.%S", sizeof cfg->opt_overwrite.datefmt);
+	g_strlcpy(cfg->opt_overwrite.datefmt, "%Y-%m-%d %H:%M.%S", sizeof cfg->opt_overwrite.datefmt);
 }
 
 #if 0
@@ -369,10 +335,6 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 		if((adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(min->gui->cur_pane->scwin))) != NULL)
 			printf("Adj: lower=%g upper=%g value=%g\n", adj->lower, adj->upper, adj->value);
 	}
-	else if((evt->keyval == GDK_s))
-		dp_focus(min->gui->cur_pane, 3);
-	else if(evt->keyval == GDK_x)
-		dp_focus(min->gui->cur_pane, 8);
 	else if(evt->keyval == GDK_z)
 		dp_unfocus(min->gui->cur_pane);
 	else if(evt->keyval == GDK_d)
@@ -398,12 +360,13 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 #endif
 
 /* 2002-02-19 -	Resolve what the initial directory is going to be, and go there. */
-static void enter_initial_dirs(MainInfo *min, const gchar *dirleft, const gchar *dirright)
+static gboolean enter_initial_dirs(MainInfo *min, const gchar *dirleft, const gchar *dirright)
 {
 	const gchar	*dir;
 	const gchar	*odir[2];
 	const gchar	*cmd[] = { "ActivateLeft", "ActivateRight" };
 	gint		i;
+	gboolean	ok = TRUE;
 
 	odir[0] = dirleft;
 	odir[1] = dirright;
@@ -413,92 +376,133 @@ static void enter_initial_dirs(MainInfo *min, const gchar *dirleft, const gchar
 		{
 			dir = min->cfg.dp_format[i].def_path;
 			if(!dir[0])
-			{
 				dir = dph_history_get_first(&min->gui->pane[i]);
-				if(dir == NULL)
-					dir = ".";
-			}
-		}
-		if(dir && *dir)
-		{
-			csq_execute(min, cmd[i]);
-			csq_execute_format(min, "DirEnter 'dir=%s'", stu_escape(dir));
 		}
+		if(dir == NULL)
+			dir = usr_get_home();
+		csq_execute(min, cmd[i]);
+		ok &= csq_execute_format(min, "DirEnter 'dir=%s'", stu_escape(dir)) != 0;
 	}
+
+	return ok;
 }
 
 /* 2003-11-25 -	An idle handler that should only trigger once, to run commands from command line. */
 static gint evt_idle_run(gpointer user)
 {
 	MainInfo	*min = user;
-	const GList	*list;
 
-	for(list = opt_arg_get("run", min->opt, min->opt_num); list != NULL; list = g_list_next(list))
-		csq_execute(min, list->data);
+	if(min->run_commands != NULL)
+	{
+		gsize	i;
+
+		for(i = 0; min->run_commands[i] != NULL; i++)
+			csq_execute(min, min->run_commands[i]);
+	}
+
 	return FALSE;		/* Causes the handler to be removed, once really is enough. */
 }
 
 int main(int argc, char *argv[])
 {
-	enum { OPT_VERSION = 1 << 0, OPT_ROOTOK = 1 << 1, OPT_NORC = 1 << 2, OPT_NOGTKRC = 1 << 3, OPT_NOFAM = 1 << 4 };
-	OptDesc	od[] = { { '\0', "version",  OPT_ARG_NONE, OPT_VERSION,	N_("Report the version to standard output, and exit.") },
-			 { '\0', "root-ok",  OPT_ARG_NONE, OPT_ROOTOK,	N_("Allows gentoo to be run by the root user. Could be dangerous.") },
-			 { '\0', "no-rc",    OPT_ARG_NONE, OPT_NORC,	N_("Do not load the ~/.gentoorc configuration file; instead, use default values.") },
-			 { '\0', "no-gtkrc", OPT_ARG_NONE, OPT_NOGTKRC,	N_("Do not load the ~/.gentoogtkrc GTK+ configuration file; instead, use system defaults.") },
-#if defined HAVE_FAM
-			 { '\0', "no-fam",   OPT_ARG_NONE, OPT_NOFAM,	N_("Do not use FAM to automatically detect changes in viewed directories.") },
-#endif
-			 { '1',  "left",     OPT_ARG_REQ,      0U, N_("Use ARG as path for the left directory pane. Overrides default (and history).") },
-			 { '2',  "right",    OPT_ARG_REQ,      0U, N_("Use ARG as path for the right directory pane. Overrides default (and history).") },
-			 { 'r',  "run",	     OPT_ARG_REQ_LIST, 0U, N_("Run ARG, a gentoo command. Done before user interaction allowed, but after configuration "
-								      "file has been read in. Can be used many times to run several commands in sequence.") }
-			};
-	guint32		opt_flags, cfg_flags = 0U;
-	GtkWidget	*box;
-	static MainInfo	min = { 0 };
+	static MainInfo	min = { NULL };
 
-	/* Make options available to idle callback that runs commands from invocation. A bit ugly. */
-	min.opt     = od;
-	min.opt_num = sizeof od / sizeof *od;
+	gboolean	show_version = FALSE;
+	gboolean	show_locale_info = FALSE;
+	gboolean	root_ok = FALSE;
+	gboolean	no_rc = FALSE, no_gtk_rc = FALSE;
+	gchar		*dir_left = NULL;
+	gchar		*dir_right = NULL;
+	GOptionEntry	option_entries[] = {
+	{ "version", 0, 0, G_OPTION_ARG_NONE, &show_version,  N_("Report the version to standard output, and exit"), NULL },
+#if defined ENABLE_NLS
+	{ "locale-info", 0, 0, G_OPTION_ARG_NONE, &show_locale_info, N_("Report internal locale details, and exit"), NULL },
+#endif
+	{ "root-ok", 0, 0, G_OPTION_ARG_NONE, &root_ok, N_("Allows gentoo to be run by the root user. Could be dangerous"), NULL },
+	{ "no-rc", 0, 0, G_OPTION_ARG_NONE, &no_rc, N_("Do not load the ~/.gentoorc configuration file; instead, use default values"), NULL },
+	{ "no-gtk-rc", 0, 0, G_OPTION_ARG_NONE, &no_gtk_rc, N_("Do not load the ~/.gentoogtkrc GTK+ configuration file; instead, use system defaults"), NULL },
+	{ "run", 'r', 0, G_OPTION_ARG_STRING_ARRAY, &min.run_commands, N_("Run COMMAND, a gentoo command. Done before user interaction allowed, but after configuration "
+			      "file has been read in. Can be used many times to run several commands in sequence"), N_("COMMAND") },
+	{ "left", '1', 0, G_OPTION_ARG_STRING, &dir_left, N_("Use DIR as path for the left directory pane. Overrides default (and history)"), N_("DIR") },
+	{ "right", '2', 0, G_OPTION_ARG_STRING, &dir_right, N_("Use DIR as path for the right directory pane. Overrides default (and history)"), N_("DIR") },
+	{ NULL }
+	};
+	gint		i;
+	GtkWidget	*box;
+	GOptionContext	*context;
+	GError		*err = NULL;
+
+	/* Must be run very early, before we start using glib functionality too much. */
+	g_thread_init(NULL);
 
 #if defined ENABLE_NLS
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	for(opt_flags = 0; opt_flags < sizeof od / sizeof *od; opt_flags++)
-		od[opt_flags].help = _(od[opt_flags].help);
-	gtk_set_locale();
+	/* Translate help texts for command line options. */
+	for(i = 0; option_entries[i].long_name != NULL; i++)
+	{
+		option_entries[i].description = _(option_entries[i].description);
+		option_entries[i].arg_description = _(option_entries[i].arg_description);
+	}
 #endif
-	/* Must be run early, before anyone changes the directory. */
-	fut_initialize();
+
+	/* Use glib's option parser, for consistency, features and just less code in general. Nice. */
+	context = g_option_context_new(_("- a graphical file manager using GTK+"));
+	g_option_context_add_main_entries(context, option_entries, PACKAGE);
+	g_option_context_add_group(context, gtk_get_option_group(TRUE));	/* Import GTK+'s options. */
+	if(!g_option_context_parse(context, &argc, &argv, &err))
+	{
+		g_print(_("Failed to parse command line options: %s\n"), err->message);
+		return EXIT_FAILURE;
+	}
 
 	/* Must be run before application-level options parsing. */
 	gtk_init(&argc, &argv);
 
-	opt_flags = opt_parse(&argc, argv, od, sizeof od / sizeof *od);
-	if(argc < 1)
-		return argc < -1 ? EXIT_FAILURE : EXIT_SUCCESS;
-	if(opt_flags & OPT_VERSION)
+	min.vfs.vfs = g_vfs_get_default();
+
+	if(show_version)
 	{
 		puts(VERSION);
 		return EXIT_SUCCESS;
 	}
-	if(geteuid() == 0 && !(opt_flags & OPT_ROOTOK))
+#if defined ENABLE_NLS
+	if(show_locale_info)
+	{
+		const gchar	*enc, *cset, **fenc;
+
+		printf("PACKAGE=\"%s\"\n", PACKAGE);
+		printf("LOCALEDIR=\"%s\"\n", LOCALEDIR);
+
+		enc = g_getenv("G_FILENAME_ENCODING");
+		if(enc == NULL)
+			enc = "(not set)";
+		printf("G_FILENAME_ENCODING=\"%s\"\n", enc);
+
+		g_get_filename_charsets(&fenc);
+		printf("Filename encoding: \"%s\"\n", (fenc != NULL && *fenc != NULL) ? fenc[0] : "(unknown)");
+
+		g_get_charset(&cset);
+		if(cset == NULL)
+			cset = "(unknown)";
+		printf("Native charset: \"%s\"\n", cset);
+		return EXIT_SUCCESS;
+	}
+#endif
+
+	if(geteuid() == 0 && !root_ok)
 	{
 		fprintf(stderr, _("%s: To run as root, envoke with the '--root-ok' option\n"), argv[0]);
 		return EXIT_FAILURE;
 	}
 
-	gtk_quit_add(1, quit_handler, &min);
-
 	dpf_initialize();
-	sze_initialize();
 	mnu_initialize();
 
 	min.cfg.flags = 0;
 	dpf_init_defaults(&min.cfg);
 	csq_init_commands(&min);
-	mnt_init_defaults(&min.cfg);
 	min.cfg.menus = mnu_menuinfo_new_default(&min);
 	btn_buttoninfo_new_default(&min, &min.cfg.buttons);
 	min.cfg.buttonlayout = btl_buttonlayout_new();
@@ -507,65 +511,64 @@ int main(int argc, char *argv[])
 	typ_init(&min.cfg);
 	min.cfg.wininfo	 = win_wininfo_new_default(&min);
 	min.cfg.ctrlinfo = ctrl_new_default(&min);
-	min.cfg.errors.dialog = FALSE;
+	min.cfg.errors.display = ERR_DISPLAY_STATUSBAR;
+	cng_initialize(&min.cfg.nag);
 	min.cfg.dir_filter = dir_filter;
 	chd_initialize(&min);
 	min.que = que_initialize();
 
 	min.ico = ico_initialize(&min);
 
-	if(!(opt_flags & OPT_NORC))
-		cfg_flags = cfg_load_config(&min);
+	min.cfg.flags = 0;
+	if(!no_rc)
+		cfg_load_config(&min);
 	if(!usr_init())
 		g_warning(_("Couldn't initialize userinfo module - username resolving won't work"));
-	if(!(opt_flags & OPT_NOGTKRC))
+	if(!no_gtk_rc)
 		load_gtk_rc(&min);
 
-	if((min.cfg.mount.mode != MNT_NEVER) && (!mnt_init(&min)))
-		g_warning(_("Couldn't initialize mount data - automounting won't work"));
-
-	if(!(opt_flags & OPT_NOFAM))
-		fam_initialize(&min);
-
 	if((box = build_gui(&min)) != NULL)
 	{
-		gchar	welcome[128];
+		dp_initialize(min.gui->pane, sizeof min.gui->pane / sizeof *min.gui->pane);
 
-		dp_initialize(min.gui->pane);
+		fam_initialize(&min);
 
 		dlg_main_window_set(GTK_WINDOW(min.gui->window));
 		dlg_position_set(min.cfg.dialogs.pos);
 #if 0
 		g_signal_connect(G_OBJECT(min.gui->window), "key_press_event", G_CALLBACK(evt_key_press), &min);
 #endif
-		min.cfg.flags = 0;
 		init_cmd_options(&min.cfg);
 
 		gtk_container_add(GTK_CONTAINER(min.gui->window), box);
 
 		dph_history_load(&min, min.gui->pane, sizeof min.gui->pane / sizeof *min.gui->pane);
 
-		gtk_window_set_policy(GTK_WINDOW(min.gui->window), TRUE, TRUE, FALSE);
+		gtk_window_set_resizable(GTK_WINDOW(min.gui->window), TRUE);
 
-		csq_execute(&min, "DpRecenter");
-
-		window_set_icon(min.gui->window);
 		win_window_show(min.gui->window);
 
-		enter_initial_dirs(&min, opt_arg_get("left", od, sizeof od / sizeof *od), opt_arg_get("right", od, sizeof od / sizeof *od));
-		if(errno == 0)
-		{
-			g_snprintf(welcome, sizeof welcome, _("gentoo v%s by Emil Brink <emil@obsession.se>"), VERSION);
-			gtk_label_set_text(GTK_LABEL(min.gui->top), welcome);
-		}
-
-		gtk_idle_add(evt_idle_run, &min);
 		err_clear(&min);
+		if(enter_initial_dirs(&min, dir_left, dir_right))
+		{
+			if(min.gui->top != NULL)
+			{
+				gchar	buf[128];
+
+				g_snprintf(buf, sizeof buf, _("gentoo v%s by Emil Brink <emil@obsession.se>"), VERSION);
+				gtk_label_set_text(GTK_LABEL(min.gui->top), buf);
+			}
+		}
+		else
+			err_show(&min);
+
+		ndl_dialog_sync_new_wait(&min, "gio-warning", _("Development Version Warning"), _("This version of gentoo is considered somewhat new and untested. There have been major changes to almost all parts of the program since the previous version. Please be a bit careful, and make sure you report any problem to the author. Thanks."));
+
+		g_idle_add(evt_idle_run, &min);
 		gtk_main();
 	}
+	chd_kill_children();
 	fam_shutdown(&min);
-	mnt_shutdown(&min);
-	fut_finalize();
 
 	return EXIT_SUCCESS;
 }
