@@ -46,12 +46,6 @@
 #include <time.h>
 #include <unistd.h>
 
-#if defined HAVE_STRUCT_STATVFS
-#include <sys/statvfs.h>
-#elif defined HAVE_STRUCT_STATFS
-#include <sys/statfs.h>
-#endif
-
 /* The various BSDs seem to have a lot in common. Let's try and use that,
 ** and save some typing in various tests below. This should probably all
 ** be replaced by proper Autoconfed stuff at some point.
@@ -66,10 +60,16 @@
 #include <sys/mount.h>
 #endif
 
-#define	GTK_DISABLE_COMPAT_H		/* No GTK+ 1.0.6-compatibility required. */
+#undef GTK_ENABLE_BROKEN		/* We can't use autoconf to undefine symbols. */
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+/* Fix for < Glib 2.20. */
+#ifndef G_GOFFSET_FORMAT
+#define G_GOFFSET_FORMAT G_GINT64_FORMAT
+#endif
+
 /* Not all systems' <mman.h> seem to define this. */
 #if !defined MAP_FAILED
 #define	MAP_FAILED	((void *) -1)
@@ -125,32 +125,35 @@ typedef struct CmdSeq		CmdSeq;
 #include "dirhistory.h"
 #include "window.h"
 #include "menus.h"
-#include "options.h"
+#include "sizeutil.h"
 
 #define	KEY_NAME_SIZE	(32)
 
 #define	CSQ_NAME_SIZE	(32)
 
-#define	FT_NAME_SIZE	(32)
-#define	FT_SUFFIX_SIZE	(32)
-#define	FT_NAMERE_SIZE	(64)
-#define	FT_FILERE_SIZE	(64)
+/* FileType-related constants. */
+enum {
+	FT_NAME_SIZE	= (32),
+	FT_SUFFIX_SIZE	= (32),
+	FT_NAMERE_SIZE	= (64),
+	FT_FILERE_SIZE	= (64),
 
-#define	FTFL_REQPERM	(1<<0)
-#define	FTFL_REQSUFFIX	(1<<1)
-#define	FTFL_NAMEMATCH	(1<<2)
-#define	FTFL_NAMEGLOB	(1<<3)
-#define	FTFL_FILEMATCH	(1<<4)
-#define	FTFL_FILEGLOB	(1<<5)
-#define	FTFL_NAMENOCASE	(1<<6)
-#define	FTFL_FILENOCASE	(1<<7)
+	FTFL_REQPERM	= (1<<0),
+	FTFL_REQSUFFIX	= (1<<1),
+	FTFL_NAMEMATCH	= (1<<2),
+	FTFL_NAMEGLOB	= (1<<3),
+	FTFL_FILEMATCH	= (1<<4),
+	FTFL_FILEGLOB	= (1<<5),
+	FTFL_NAMENOCASE	= (1<<6),
+	FTFL_FILENOCASE	= (1<<7),
 
-#define	FTPM_SETUID	(1<<0)
-#define	FTPM_SETGID	(1<<1)
-#define	FTPM_STICKY	(1<<2)
-#define	FTPM_READ	(1<<3)
-#define	FTPM_WRITE	(1<<4)
-#define	FTPM_EXECUTE	(1<<5)
+	FTPM_SETUID	= (1<<0),
+	FTPM_SETGID	= (1<<1),
+	FTPM_STICKY	= (1<<2),
+	FTPM_READ	= (1<<3),
+	FTPM_WRITE	= (1<<4),
+	FTPM_EXECUTE	= (1<<5),
+};
 
 typedef struct {			/* Used to map stat() info onto names. */
 	gchar	name[FT_NAME_SIZE];		/* Human-readable name of filetype (e.g. "GIF image", "MP3 song"). */
@@ -172,25 +175,23 @@ typedef struct {			/* Used to map stat() info onto names. */
 
 typedef enum {	DPC_NAME, DPC_SIZE, DPC_BLOCKS, DPC_BLOCKSIZE, DPC_MODENUM, DPC_MODESTR,
 		DPC_NLINK, DPC_UIDNUM, DPC_UIDSTR, DPC_GIDNUM, DPC_GIDSTR, DPC_DEVICE, DPC_DEVMAJ, DPC_DEVMIN,
-		DPC_ATIME, DPC_MTIME, DPC_CTIME,
+		DPC_ATIME, DPC_MTIME, DPC_CRTIME, DPC_CHTIME,
 		DPC_TYPE, DPC_ICON,
+		DPC_URI_NOFILE,
 		DPC_NUM_TYPES } DPContent;
 
 typedef struct {
-	gint	show_type : 1;			/* Append type-character (from "@ / * | =") to names? */
-	gint	show_linkname : 1;		/* Append " -> destination" on symbolic links? */
+	guint	show_type : 1;			/* Append type-character (from "@ / * | =") to names? */
+	guint	show_linkname : 1;		/* Append " -> destination" on symbolic links? */
 } DC_Name;
 
-typedef enum {  DPSIZE_NATIVE = 0, DPSIZE_BYTES, DPSIZE_KILOBYTES, DPSIZE_MEGABYTES, DPSIZE_GIGABYTES,
-		DPSIZE_SMART  = 32 } DPSizeType;
-
 typedef struct {
-	DPSizeType	type;
-	gint		ticks : 1;
+	SzUnit		unit;
+	guint		ticks : 1;
 	gchar		tick;
 	gint		digits;
 	gchar		dformat[DP_FORMAT_SIZE];	/* Hidden from user, not saved, built by config code. */
-	gint		dir_show_fs_size : 1;		/* Show filesystem size for directories? */
+	guint		dir_show_fs_size : 1;		/* Show filesystem size for directories? */
 } DC_Size;
 
 typedef struct {
@@ -206,7 +207,7 @@ typedef union {
 	DC_Size	size;
 	DC_Fmt	blocks, blocksize;
 	DC_Fmt	mode, nlink, uidnum, gidnum, device, devmaj, devmin;
-	DC_Time	a_time, m_time, c_time;		/* Accessed, modified & changed, right? */
+	DC_Time	a_time, m_time, cr_time, ch_time;
 } DpCExtra;
 
 typedef struct {
@@ -224,13 +225,11 @@ typedef struct {
 	SortMode	mode;			/* Controls placement of directories. */
 	gboolean	invert;			/* If set, we sort backwards (Z-A). */
 	gboolean	nocase;			/* Set for case-insensitive string comparisons. Lame! */
-	gboolean	numerical;		/* Special numerical mode (way cool)? */
 } DPSort;
 
 typedef enum { SBP_IGNORE = 0, SBP_LEFT, SBP_RIGHT } SBarPos;
 
-/* Click mode -- determines if (and how) we alter the CONTROL state for pane clicks. */
-typedef enum { CTM_IGNORE = 0, CTM_SET, CTM_RESET, CTM_TOGGLE } CtrlMode;
+#define	FONT_MAX	128
 
 typedef struct {
 	guint		num_columns;
@@ -242,15 +241,11 @@ typedef struct {
 	gboolean	scrollbar_always;	/* Set to always show scrollbar, regardless of # of entries. */
 	gboolean	huge_parent;		/* Set to enable huge, tall, Opus-like parent button. */
 	SBarPos		sbar_pos;		/* Position of scrollbar. */
-	CtrlMode	ctrl_mode;		/* Controls altering of the Control key state for pane clicks. */
+	gboolean	set_font;		/* Set to override GTK+'s default font. */
+	gchar		font_name[FONT_MAX];	/* Last-set font name, remains valid even when set_font == FALSE. */
+	gboolean	rubber_banding;		/* Set to enable GTK+'s "rubberbanding" selection. */
 } DPFormat;
 
-typedef enum { DPCOL_PANE_SELECTED = 0, DPCOL_FOCUS_UNSELECTED, DPCOL_FOCUS_SELECTED, DPCOL_NUM } DpCol;
-
-typedef struct {
-	GdkColor	color[DPCOL_NUM];
-} DPColors;
-
 typedef enum { DPORIENT_HORIZ = 0, DPORIENT_VERT } DpOrient;
 typedef enum { DPSPLIT_FREE = 0,   DPSPLIT_RATIO, DPSPLIT_ABS_LEFT, DPSPLIT_ABS_RIGHT } DpSplit;
 
@@ -269,18 +264,22 @@ typedef struct {
 /* ----------------------------------------------------------------------------------------- */
 
 /* General command row flags. */
-#define	CGF_RUNINBG		(1<<0)		/* Run in command in background? */
-#define	CGF_KILLPREV		(1<<1)		/* Kill previous instance of program (only background)? */
-#define	CGF_GRABOUTPUT		(1<<2)		/* Capture the output into a special window? */
-#define	CGF_SURVIVE		(1<<3)		/* Survive when gentoo quits (only background)? */
+enum {
+	CGF_RUNINBG = (1<<0),		/* Run in command in background? */
+	CGF_KILLPREV = (1<<1),		/* Kill previous instance of program (only background)? */
+	CGF_GRABOUTPUT = (1<<2),	/* Capture the output into a special window? */
+	CGF_SURVIVE = (1<<3)		/* Survive when gentoo quits (only background)? */
+};
 
 /* These flags are for the before- and after-flag fields. */
-#define	CBAF_RESCAN_SOURCE	(1<<0)		/* Rescan the source dir pane? */
-#define	CBAF_RESCAN_DEST	(1<<1)
-#define	CBAF_CD_SOURCE		(1<<2)		/* These save some config work. */
-#define	CBAF_CD_DEST		(1<<3)
-#define CBAF_REQSEL_SOURCE	(1<<4)		/* Command won't run if there's no source selection when envoked. */
-#define CBAF_REQSEL_DEST	(1<<5)		/* Command won't run if there's no destination selection when envoked. */
+enum {
+	CBAF_RESCAN_SOURCE = (1<<0),		/* Rescan the source dir pane? */
+	CBAF_RESCAN_DEST = (1<<1),
+	CBAF_CD_SOURCE = (1<<2),		/* These save some config work. */
+	CBAF_CD_DEST = (1<<3),
+	CBAF_REQSEL_SOURCE = (1<<4),		/* Command won't run if there's no source selection when envoked. */
+	CBAF_REQSEL_DEST = (1<<5)		/* Command won't run if there's no destination selection when envoked. */
+};
 
 typedef struct {		/* Extra info for external commands. */
 	guint32	gflags;			/* General flags. */
@@ -323,7 +322,7 @@ typedef struct {
 } ButtonInfo;
 
 typedef struct {			/* Options for overwrite-confirmation dialog module. */
-	gint	show_info : 1;			/* Show info (sizes & dates) for conflicting files? */
+	guint	show_info : 1;			/* Show info (sizes & dates) for conflicting files? */
 	gchar	datefmt[DP_DATEFMT_SIZE];	/* How to format dates? */
 } OptOverwrite;
 
@@ -333,7 +332,7 @@ typedef struct {			/* Info about files hidden from user. Not ignored, just hidde
 	HMode	mode;
 	gchar	hide_re_src[MAXNAMLEN];		/* Regular expression; matches are hidden if mode == HIDE_REGEXP. */
 	regex_t	*hide_re;			/* Compiled version of the RE. */
-	gint	no_case : 1;			/* Ignore case in the RE? */
+	guint	no_case : 1;			/* Ignore case in the RE? */
 } HideInfo;
 
 typedef enum {	PTID_ICON = 0, PTID_GTKRC, PTID_FSTAB, PTID_MTAB,
@@ -344,30 +343,25 @@ typedef struct {				/* Miscellanous paths. */
 	HideInfo	hideinfo;			/* Info about which files should be hidden. */
 } PathInfo;
 
-/* This controls how often we do mounts, if ever. The levels are:
-** MNT_NEVER	Disable all mounting.
-** MNT_ALWAYS	Does a mount attempt on every directory scan. Might be slow.
-*/
-typedef enum { MNT_NEVER = 0, MNT_ALWAYS }	MntMode;
-
-typedef struct {			/* Configuration info for automounting. */
-	MntMode	mode;				/* Global mounting mode. [MNT_NEVER] */
-	gint	check_nlink : 1;		/* Require nlink == 2 before mounting? [TRUE] */
-	gint	show_stderr : 1;		/* Show stderr of mount/umount command? [TRUE] */
-	gint	umount_exit : 1;		/* Unmount all mounted dirs at exit? [TRUE] */
-	gchar	cmd_mount[MAXNAMLEN];		/* Full name to command to use for mounting. ["/bin/mount"] */
-	gchar	cmd_umount[MAXNAMLEN];		/* This command is used for unmounting. ["/bin/umount"] */
-} MountInfo;
-
 typedef struct {			/* Configuration information for dialogs. */
 	GtkWindowPosition	pos;		/* How should dialogs be positioned? */
 } DialogInfo;
 
+typedef enum {
+	ERR_DISPLAY_STATUSBAR,
+	ERR_DISPLAY_TITLEBAR,
+	ERR_DISPLAY_DIALOG
+} ErrDisplay;
+
 typedef struct {
-	gboolean	dialog;			/* Use dialog for errors? */
+	ErrDisplay	display;		/* How should error (and status messages) be shown? */
 	gboolean	beep;			/* Do a gdk_beep() on error? */
 } ErrInfo;
 
+typedef struct {
+	GHashTable	*ignored;		/* Keyed on 'tag', if present ignore the dialog. */
+} NagInfo;
+
 #define	CFLG_CHANGED	(1<<0)		/* Config has changed. Set on "OK", cleared by cfg_save_all(). */
 
 typedef struct {			/* Holds all configuration info. */
@@ -375,7 +369,6 @@ typedef struct {			/* Holds all configuration info. */
 	GList		*type;			/* List of types (FType structures). */
 	StyleInfo	*style;			/* Opaque style container. */
 	DPFormat	dp_format[2];
-	DPColors	dp_colors;		/* Global color information used to render dirpanes. */
 	DPPaning	dp_paning;		/* Controls how the paned view acts. */
 	DPHistory	dp_history;		/* History list option. */
 	OptOverwrite	opt_overwrite;
@@ -384,19 +377,23 @@ typedef struct {			/* Holds all configuration info. */
 	ButtonLayout	*buttonlayout;
 	CmdInfo		commands;
 	PathInfo	path;
-	MountInfo	mount;
 	WinInfo		*wininfo;
 	DialogInfo	dialogs;
 	CtrlInfo	*ctrlinfo;
 	ErrInfo		errors;
 	gboolean	(*dir_filter)(const gchar *name);
+	NagInfo		nag;
 } CfgInfo;
 
 typedef struct GuiInfo	GuiInfo;
 
+typedef struct {
+	GVfs		*vfs;
+} VfsInfo;
+
 struct MainInfo {				/* gentoo's single most central data structure. */
-	OptDesc		*opt;
-	gsize		opt_num;
+	gchar		**run_commands;		/* From the command line --run option, kept first for initializer in main(). */
+	VfsInfo		vfs;
 	CfgInfo		cfg;
 	GuiInfo		*gui;
 	IconInfo	*ico;
@@ -412,43 +409,9 @@ typedef struct {
 	GtkEntryCompletion	*compl;			/* A GTK+ 2.0 completion object, attached to the pane's path entry. */
 } PathComplete;
 
-/* These macros are used to access fields of a DirRow structure. */
-#define	DP_ROW_NAME(r)		((r)->dr_names.disk)
-#define	DP_ROW_NAMED(r)		((r)->dr_names.display)
-#define	DP_ROW_LSTAT(r)		((r)->dr_lstat)
-#define	DP_ROW_STAT(r)		(*((r)->dr_stat))
-#define	DP_ROW_LINKNAME(r)	((r)->dr_linknames.disk)
-#define	DP_ROW_LINKNAMED(r)	((r)->dr_linknames.display)
-#define	DP_ROW_FLAGS(r)		((r)->dr_flags)
-#define	DP_ROW_FLAGS_SET(r,f)	(DP_ROW_FLAGS(r) |= (f))
-#define	DP_ROW_FLAGS_CLR(r,f)	(DP_ROW_FLAGS(r) &= ~(f))
-#define	DP_ROW_FLAGS_CHK(r,f)	((DP_ROW_FLAGS(r) & (f)) == (f))
-#define	DP_ROW_TYPE(r)		((r)->dr_type)
-#define	DP_ROW_INDEX(p,r)	((r) - (p)->dir.row)
-
-#define	DP_ROW_SET_STAT(r,s)	(((r)->dr_stat) = (s))
-#define	DP_ROW_GET_STAT(r)	((r)->dr_stat)
-#define	DP_ROW_SET_LINKNAME(r,n) ((r)->dr_linknames.disk = (n))
-#define	DP_ROW_SET_LINKNAMED(r,n) ((r)->dr_linknames.display = (n))
-
-/* These macros are to be used when accessing a DirRow through a selection list, which
-** is the most frequent case. The 's' parameter is the SList iterator.
-*/
-#define	DP_SEL_ROW(s)		((DirRow *) (s)->data)
-#define	DP_SEL_NAME(s)		(DP_ROW_NAME(DP_SEL_ROW(s)))
-#define	DP_SEL_NAMED(s)		(DP_ROW_NAMED(DP_SEL_ROW(s)))
-#define	DP_SEL_LSTAT(s)		(DP_ROW_LSTAT(DP_SEL_ROW(s)))
-#define	DP_SEL_STAT(s)		(DP_ROW_STAT(DP_SEL_ROW(s)))
-#define	DP_SEL_LINKNAME(s)	(DP_ROW_LINKNAME(DP_SEL_ROW(s)))
-#define	DP_SEL_FLAGS(s)		(DP_ROW_FLAGS(DP_SEL_ROW(s)))
-#define	DP_SEL_FLAGS_SET(s,f)	(DP_ROW_FLAGS_SET(DP_SEL_ROW(s), (f)))
-#define	DP_SEL_FLAGS_CLR(s,f)	(DP_ROW_FLAGS_CLR(DP_SEL_ROW(s), (f)))
-#define	DP_SEL_FLAGS_CHK(s,f)	(DP_ROW_FLAGS_CHK(DP_SEL_ROW(s), f))
-#define	DP_SEL_TYPE(s)		(DP_ROW_TYPE(DP_SEL_ROW(s)))
-#define	DP_SEL_INDEX(p,s)	(DP_ROW_INDEX((p), DP_SEL_ROW(s)))
-
-enum {	DPRF_HAS_SIZE = 1 << 0,			/* Set when a row has a "real" size (not set for directories unless GetSize:d). */
-	DPRF_LINK_EXISTS = 1 << 1		/* Set when a symlink's target exists. */
+enum {	DPRF_HAS_SIZE	 = 1 << 0,		/* Set when a row has a "real" size (not set for directories unless GetSize:d). */
+	DPRF_LINK_EXISTS = 1 << 1,		/* Set when a symlink's target exists. */
+	DPRF_LINK_TO_DIR = 1 << 2		/* Set when symlink points at (valid) directory. Implies DPRF_LINK_EXISTS. */
 };
 
 typedef struct {			/* A filename, in both on-disk and display formats. */
@@ -465,30 +428,30 @@ typedef struct {			/* Representation of a single line in a directory (a file, ty
 	const FType	*dr_type;		/* Type information. */
 } DirRow;
 
+typedef GtkTreeIter	DirRow2;
+
 typedef struct {
 	guint		num_dirs, num_files;
-	guint64		num_bytes, num_blocks;
+	guint64		num_bytes;
 } SelInfo;
 
 typedef struct {			/* Some trivial file system information. Updated on rescan. */
 	gboolean	valid;			/* Set if the structure's contents are valid. */
-#if defined HAVE_STRUCT_STATVFS
-	struct statvfs	stat;			/* Holds information about filesystem. */
-#elif defined HAVE_STRUCT_STATFS || defined GENTOO_ON_BSD
-	struct statfs	stat;
-#endif
+	guint64		fs_size;		/* Size of filesystem, in bytes. */
+	guint64		fs_free;		/* Free bytes in this filesystem. */
 } FsInfo;
 
+#define	URI_MAX		(2000)			/* This should only rarely be used, and hopefully ease out once GIO gets more mature. */
+
 typedef struct {
-	gchar		path[PATH_MAX];		/* Contents apply to this path. */
+	gchar		path[URI_MAX];		/* Contents apply to this path. */
 	gchar		*pathd;			/* Dynamically allocated display version of the path. Beware! */
+	GtkListStore	*store;			/* Actual ListStore holding the pane's contents. Insert and clear here. */
+	GFile		*root;			/* A GFile representing the current location shown. */
+	gboolean	is_local;		/* Is the root considered local? Used to control 'file' etc. */
 	guint		num_rows;		/* Number of valid rows. */
 	guint		tot_dirs, tot_files;	/* Total number of entries in directory. */
-	guint64		tot_bytes, tot_blocks;	/* Sum of all sizes (in bytes & blocks). */
-	DirRow		*row;			/* A bunch of rows. */
-	gsize		row_alloc;		/* Number of allocated rows. */
-	gchar		*name;			/* This holds name info (filenames and link names). */
-	gsize		name_alloc, name_use;	/* Allocated and used name bytes. */
+	guint64		tot_bytes;		/* Sum of all sizes, in bytes. */
 	struct stat	*stat;			/* Info about link targets. */
 	gsize		stat_alloc, stat_use;	/* Allocated and used link stat structs. */
 	SelInfo		sel;
@@ -502,11 +465,13 @@ struct DirPane {
 	guint		index;			/* Index of *this* pane, in the grand scheme of things. */
 	GtkWidget	*vbox;
 	GtkWidget	*notebook;		/* Notebook holding (normally) path widgetry. */
+	GtkWidget	*parent;		/* The common parent button. */
+	GtkWidget	*hparent;		/* The "huge" parent button, if enabled in config. */
 	GtkWidget	*path;			/* GTK+ 2.0 GtkComboBox showing path and path history. */
 	PathComplete	complete;		/* Information for completing the path. */
 	GtkWidget	*hide;			/* A toggle button showing (and controlling) hide status. Focus target! */
 	GtkWidget	*scwin;			/* Scrolling window for clist to live in. */
-	GtkCList	*list;			/* The actual clist that shows all stuff. */
+	GtkWidget	*view;			/* GtkTreeView for viewing the pane. */
 	GtkWidget	*menu_top;		/* The top-level menu, typically shown by right-clicking. */
 	GtkWidget	*mitem_action;		/* The item in the top-level menu that contains the actions submenu. */
 	GtkWidget	*menu_action;		/* The menu containing the intersection of all available actions on the selection. */
@@ -517,6 +482,7 @@ struct DirPane {
 	gint		dbclk_row;		/* The row that was double clicked, or -1. */
 	gint		focus_row;		/* The last row to be rendered focused, or -1. */
 	gint		old_focus_row;		/* Used to remember focus when pane is deactivated. */
+	guint		sig_sel_changed;	/* Signal for the "changed" event on the view's GtkTreeSelection. */
 	guint		sig_path_activate;	/* Signal for the "activate" event on path entry combo box. */
 	guint		sig_path_changed;	/* Signal for the "changed" event on the path entry combo box. */
 };
@@ -527,8 +493,6 @@ struct GuiInfo {
 	GtkWidget	*window;
 	guint		sig_main_configure;
 	guint		sig_main_delete;
-	gboolean	win_configured;
-	GtkTooltips	*main_tt;		/* Tooltips for main window. Used by dirpane widgets. */
 	KbdContext	*kbd_ctx;		/* Keyboard context, for shortcuts. */
 	GtkWidget	*vbox;			/* Vbox that contains entire GUI. */
 	GtkWidget	*top;			/* A label showing status (selections, free space, etc). */
@@ -548,5 +512,6 @@ struct GuiInfo {
 /* These are defined in the main "gentoo.c" module, but really shouldn't be. Until they
 ** get a module of their own, they need to be prototyped like this. :(
 */
+extern void	rebuild_top(MainInfo *min);
 extern void	rebuild_middle(MainInfo *min);
 extern void	rebuild_bottom(MainInfo *min);
