@@ -48,21 +48,27 @@ GtkWidget * gui_arrow_button_new(GtkArrowType at)
 	return but;
 }
 
+/* 2010-06-05 -	Wrapper since I can't get gtk_button_new_from_stock() to behave the way I like. Weird. */
+GtkWidget * gui_stock_button_new(const gchar *stock_id)
+{
+	GtkWidget	*btn, *img;
+
+	btn = gtk_button_new();
+	img = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_MENU);
+	gtk_button_set_image(GTK_BUTTON(btn), img);
+
+	return btn;
+}
+
 /* 1999-04-26 -	Create a button with a "details" icon on it. This is typically a magnifying
 **		class. This button is a graphical replacement for a button labeled "...".
 ** 2009-04-23 -	Happy ten year anniversary, gui_details_button_new()! W00t! Rewritten to use
 **		GTK+ 2.0 stock image, GTK_STOCK_FIND is what we use. It used to be a magnifying
 **		glass, but in current GTK+ it's a pair of binoculars. Oh well.
 */
-GtkWidget * gui_details_button_new(GdkWindow *win)
+GtkWidget * gui_details_button_new(void)
 {
-	GtkWidget	*btn, *img;
-
-	btn = gtk_button_new();
-	/* This is a bit of a hack; even the MENU size is not quite small enough. */
-	img = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
-	gtk_button_set_image(GTK_BUTTON(btn), img);
-	return btn;
+	return gui_stock_button_new(GTK_STOCK_FIND);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -78,8 +84,8 @@ GSList * gui_radio_group_new(guint num, const gchar **label, GtkWidget *but[])
 	for(i = 0; i < num; i++)
 	{
 		but[i] = gtk_radio_button_new_with_label(list, _(label[i]));
-		list = gtk_radio_button_group(GTK_RADIO_BUTTON(but[i]));
-		gtk_object_set_user_data(GTK_OBJECT(but[i]), GUINT_TO_POINTER(i));
+		list = gtk_radio_button_get_group(GTK_RADIO_BUTTON(but[i]));
+		g_object_set_data(G_OBJECT(but[i]), "user", GUINT_TO_POINTER(i));
 	}
 	return list;
 }
@@ -131,7 +137,7 @@ gboolean gui_progress_update(gpointer anchor, gfloat value, const gchar *obj)
 
 	if(!pbar->terminate)
 	{
-		gtk_progress_bar_update(GTK_PROGRESS_BAR(pbar->bar), value);
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar->bar), value);
 		if((obj != NULL) && (pbar->obj != NULL))
 			gtk_label_set_text(GTK_LABEL(pbar->obj), obj);
 		gui_events_flush();
@@ -161,7 +167,7 @@ void gui_progress_end(gpointer anchor)
 **		determine which item was selected, each item will have its index (from 0)
 **		set as object's user data. Not pretty, but still simple and useful.
 */
-GtkWidget * gui_build_menu(const gchar *text[], GtkSignalFunc func, gpointer user)
+GtkWidget * gui_build_menu(const gchar *text[], GCallback func, gpointer user)
 {
 	GtkWidget	*menu, *item;
 	guint		i;
@@ -172,15 +178,63 @@ GtkWidget * gui_build_menu(const gchar *text[], GtkSignalFunc func, gpointer use
 		item = gtk_menu_item_new_with_label(_(text[i]));
 		if(func != NULL)
 		{
-			gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(i));
+			g_object_set_data(G_OBJECT(item), "user", GUINT_TO_POINTER(i));
 			g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(func), user);
 		}
-		gtk_menu_append(GTK_MENU(menu), item);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		gtk_widget_show(item);
 	}
 	return menu;
 }
 
+
+/* 2010-05-10 -	Trampoline for the gui_build_combo_box()-constructed widgets. Computes the
+**		index that was selected, and hands that to the user-supplied callback.
+*/
+static void evt_combo_changed(GtkWidget *wid, gpointer user)
+{
+	const gint	item = gtk_combo_box_get_active(GTK_COMBO_BOX(wid));
+
+	if(item >= 0)
+	{
+		GCallback	func = g_object_get_data(G_OBJECT(wid), "callback");
+
+		if(func != NULL)
+			((void(*)(GtkWidget*, guint, gpointer))func)(wid, item, user);
+	}
+}
+
+/* 2010-05-09 -	Merry birthday, me! :D This is a replacement for the old gui_build_menu().
+**		It builds a text-based GtkComboBox, and calls the given function like so:
+**			void callback(GtkWidget *combo, guint index, gpointer user);
+**		Simple, and friendly.
+*/
+GtkWidget * gui_build_combo_box(const gchar *text[], GCallback func, gpointer user)
+{
+	GtkWidget	*cbox;
+	gulong		handler;
+
+	cbox = gtk_combo_box_text_new();
+	g_object_set_data(G_OBJECT(cbox), "callback", func);
+	for(; *text != NULL; text++)
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(cbox), _(*text));
+	handler = g_signal_connect(G_OBJECT(cbox), "changed", G_CALLBACK(evt_combo_changed), user);
+	g_object_set_data(G_OBJECT(cbox), "handler", GINT_TO_POINTER(handler));
+
+	return cbox;
+}
+
+/* 2010-11-02 -	Block the signal handler. Since it's a bit encapsulated, a function here is needed. */
+void gui_combo_box_set_blocked(GtkWidget *widget, gboolean blocked)
+{
+	const gulong	handler = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "handler"));
+
+	if(blocked)
+		g_signal_handler_block(G_OBJECT(widget), handler);
+	else
+		g_signal_handler_unblock(G_OBJECT(widget), handler);
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 GuiHandlerGroup * gui_handler_group_new(void)
@@ -241,18 +295,26 @@ void gui_handler_group_unblock(const GuiHandlerGroup *g)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 2008-07-04 -	Flush pending GTK+ events. This is handy as a separate function, even though
-**		it's only two lines of actual code. It seems that on some systems (Ubuntu),
-**		GTK+ smashes errno when running the event flush, and this encapsulation
-**		gives us a chance of handling that easily.
+void gui_set_main_title(MainInfo *min, const gchar *title)
+{
+	gchar	buf[256];
+
+	if(title != NULL && *title != '\0')
+	{
+		g_snprintf(buf, sizeof buf, "gentoo - %s", title);
+		win_window_set_title(min->gui->window, buf);
+	}
+	else
+		win_window_set_title(min->gui->window, "gentoo");
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
+/* 2008-07-04 -	Flush pending GTK+ events.
+** 2010-03-04 -	Comments trimmed, legacy code removed. Just the events, thanks.
 */
 void gui_events_flush(void)
 {
-	gint	old_errno = errno;
-
-	/* Due to side-effects, this can clobber errno, so save it. */
 	while(gtk_events_pending())
 		gtk_main_iteration();
-
-	errno = old_errno;
 }
