@@ -1,12 +1,14 @@
 /*
+** Custom button widget, that supports having a secondary click-function associated with being
+** clicked by the middle mouse button. Very much inspired by Directory Opus on the Amiga.
 **
+** Original GTK+ 1.2.x version written by Johan Hanson, re-written for GTK+ 2.0 by Emil Brink.
 */
 
 #include <stdio.h>
 #include <stdlib.h>
 
-/* ODMultiButton is 2.x pure, like freshly fallen snow. */
-#define GTK_DISABLE_DEPRECATED
+#include "config.h"
 
 #include <gtk/gtk.h>
 
@@ -73,8 +75,8 @@ static void od_multibutton_set_page(GtkWidget *widget, guint index)
 		}
 		if(p != NULL)
 		{
-			if(GTK_WIDGET_STATE(widget) != GTK_WIDGET_STATE(p))
-				gtk_widget_set_state(p, GTK_WIDGET_STATE(widget));
+			if(gtk_widget_get_state(widget) != gtk_widget_get_state(p))
+				gtk_widget_set_state(p, gtk_widget_get_state(widget));
 			gtk_widget_show(p);
 			if(gtk_widget_get_parent(p) != NULL)
 				gtk_widget_reparent(p, widget);
@@ -86,7 +88,7 @@ static void od_multibutton_set_page(GtkWidget *widget, guint index)
 		}
 		mb->last_index = index;
 	}
-	if(GTK_WIDGET_DRAWABLE(widget))
+	if(gtk_widget_is_drawable(widget))
 		gtk_widget_queue_draw(widget);
 }
 
@@ -122,10 +124,8 @@ static void od_multibutton_size_request(GtkWidget *widget, GtkRequisition *req)
 
 	mb = OD_MULTIBUTTON(widget);
 
-	req->width  = 2 * (gtk_container_get_border_width(GTK_CONTAINER(widget)) + 1 +
-			gtk_widget_get_style(widget)->xthickness);
-	req->height = 2 * (gtk_container_get_border_width(GTK_CONTAINER(widget)) + 1 +
-			gtk_widget_get_style(widget)->ythickness);
+	req->width  = 2 * (gtk_container_get_border_width(GTK_CONTAINER(widget)) + 1 + gtk_widget_get_style(widget)->xthickness);
+	req->height = 2 * (gtk_container_get_border_width(GTK_CONTAINER(widget)) + 1 + gtk_widget_get_style(widget)->ythickness);
 
 	/* Included for completeness, ODMultiButton can't actually be focused. */
 	gtk_widget_style_get(GTK_WIDGET(widget), "focus-line-width", &focus_width, "focus-padding", &focus_pad, NULL);
@@ -142,11 +142,9 @@ static void od_multibutton_size_request(GtkWidget *widget, GtkRequisition *req)
 */
 static void od_multibutton_paint_dog_ear(GtkWidget *widget, GdkRectangle *area)
 {
-	GtkStateType	state;
+	const GtkStateType	state = gtk_widget_get_state(widget);
 
-	state = GTK_WIDGET_STATE(widget);
-
-	if(GTK_WIDGET_DRAWABLE(widget) && state != GTK_STATE_ACTIVE)
+	if(gtk_widget_is_drawable(widget) && state != GTK_STATE_ACTIVE)
 	{
 		GtkStyle	*style = gtk_widget_get_style(widget);
 		const guint	EARSIZE = 5;
@@ -209,7 +207,7 @@ static void od_multibutton_paint_foreground(GtkWidget *widget, GdkRectangle *are
 	g_return_if_fail(area != NULL);
 	g_return_if_fail(OD_IS_MULTIBUTTON(widget));
 
-	if(GTK_WIDGET_DRAWABLE(widget))
+	if(gtk_widget_is_drawable(widget))
 	{
 		ODMultiButton	*mb = OD_MULTIBUTTON(widget);
 
@@ -321,7 +319,13 @@ static void od_multibutton_init(ODMultiButton *mb)
 {
 	guint	i;
 
+#if GTK_CHECK_VERSION(2, 2, 0)
+	gtk_widget_set_can_focus(GTK_WIDGET(mb), FALSE);
+	gtk_widget_set_can_default(GTK_WIDGET(mb), FALSE);
+	gtk_widget_set_receives_default(GTK_WIDGET(mb), FALSE);
+#else
 	GTK_WIDGET_UNSET_FLAGS(mb, GTK_CAN_FOCUS | GTK_CAN_DEFAULT | GTK_RECEIVES_DEFAULT);
+#endif
 	for(i = 0; i < sizeof mb->page / sizeof *mb->page; i++)
 	{
 		mb->page[i].widget = NULL;
@@ -449,7 +453,7 @@ void od_multibutton_set_text(ODMultiButton *mb, guint index, const gchar *text,
 			od_multibutton_reset_label(mb, index, GTK_LABEL(w), text, bg, fg);
 			if(GTK_WIDGET(mb)->parent)
 				gtk_widget_queue_resize(GTK_WIDGET(mb));
-			if(GTK_WIDGET_DRAWABLE(mb))
+			if(gtk_widget_is_drawable(GTK_WIDGET(mb)))
 				gtk_widget_queue_draw(GTK_WIDGET(mb));
 		}
 	}
