@@ -1,8 +1,8 @@
 /*
- ** 1999-12-23 -	Hm, perhaps this is an unfortunate choice of module title. It might bloat,
- **		or something. :) Anyway, the purpose of this module is to help handle
- **		(toplevel) windows, specifically remembering their sizes and positions.
- */
+** 1999-12-23 -	Hm, perhaps this is an unfortunate choice of module title. It might bloat,
+**		or something. :) Anyway, the purpose of this module is to help handle
+**		(toplevel) windows, specifically remembering their sizes and positions.
+*/
 
 #include "gentoo.h"
 
@@ -10,6 +10,7 @@
 #include <stdlib.h>
 
 #include "guiutil.h"
+#include "graphics/icon_gentoo_small.xpm"
 
 #include "window.h"
 
@@ -17,28 +18,28 @@
 
 typedef struct
 {
-	guint32 id; /* Unique identifier for this windata. */
-	WinType type; /* What type of window? */
-	gboolean persistent; /* Makes close() actually do a hide(). */
-	const gchar *title; /* Text for window's title bar. */
-	const gchar *label; /* Short descriptive label (like "main", "config" etc). */
-	gboolean pos_grab;
-	gboolean size_grab;
-	/* Data below here is stored in config file. */
-	gint x, y; /* Window's opening position. */
-	gint width, height; /* The desired size of this window. */
-	gboolean pos_use; /* Should we set use the recorded position? */
-	gboolean pos_update; /* Update recorded position when window closes? */
-	gboolean size_use; /* Use recorded size? */
-	gboolean size_update; /* Update recorded size with current size on close? */
+	guint32		id;			/* Unique identifier for this windata. */
+	WinType		type;			/* What type of window? */
+	gboolean	persistent;		/* Makes close() actually do a hide(). */
+	const gchar	*title;			/* Text for window's title bar. */
+	const gchar	*label;			/* Short descriptive label (like "main", "config" etc). */
+	gboolean	pos_grab;
+	gboolean	size_grab;
+	/* Data below here is stored in configuration file. */
+	gint		x, y;			/* Window's opening position. */
+	gint		width, height;		/* The desired size of this window. */
+	gboolean	pos_use;		/* Should we set use the recorded position? */
+	gboolean	pos_update;		/* Update recorded position when window closes? */
+	gboolean	size_use;		/* Use recorded size? */
+	gboolean	size_update;		/* Update recorded size with current size on close? */
 } WinDef;
 
 struct WinInfo
 {
-	MainInfo *min;			/* The handiness of this pointer cannot be overstated. */
-	GList *windows;			/* List of WinDef:s as per above. There's no rush. */
-	gint	border_width;		/* User's idea of her window manager's border size. Hackish. */
-	gint	border_height;
+	MainInfo	*min;			/* The handiness of this pointer cannot be overstated. */
+	GList		*windows;		/* List of WinDef:s as per above. There's no rush. */
+	gint		border_width;		/* User's idea of her window manager's border size. Hackish. */
+	gint		border_height;
 };
 
 /* ----------------------------------------------------------------------------------------- */
@@ -80,14 +81,14 @@ WinInfo * win_wininfo_new_default(MainInfo *min)
 
 	win_window_new(wi, WIN_CONFIG, WIN_TYPE_COMPLEX_DIALOG, TRUE,
 			_("Configure gentoo"), "config");
-	win_window_pos_use_set(wi, WIN_CONFIG, TRUE);
+	win_window_pos_use_set(wi, WIN_CONFIG, FALSE);
 	win_window_size_set(wi, WIN_CONFIG, -1, 464);
 	win_window_size_use_set(wi, WIN_CONFIG, TRUE);
 	win_window_size_update_set(wi, WIN_CONFIG, TRUE);
 
 	win_window_new(wi, WIN_TEXTVIEW, WIN_TYPE_SIMPLE_TOPLEVEL, FALSE,
 			_("Text Viewer"), "textview");
-	win_window_pos_use_set(wi, WIN_TEXTVIEW, TRUE);
+	win_window_pos_use_set(wi, WIN_TEXTVIEW, FALSE);
 	win_window_size_set(wi, WIN_TEXTVIEW, 640, 480);
 	win_window_size_use_set(wi, WIN_TEXTVIEW, TRUE);
 	win_window_size_update_set(wi, WIN_TEXTVIEW, TRUE);
@@ -147,38 +148,36 @@ static void evt_boolean_clicked(GtkWidget *wid, gpointer user)
 	gboolean *modified;
 
 	*flag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
-	if((modified = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
+	if((modified = g_object_get_data(G_OBJECT(wid), "modified")) != NULL)
 		*modified = TRUE;
 }
 
-static void evt_integer_changed(GtkWidget *wid, gpointer user)
+static void evt_integer_changed(GtkAdjustment *adj, gpointer user)
 {
 	gint *value = user;
 	gboolean *modified;
 
-	*value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wid));
-	if((modified = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
+	*value = gtk_adjustment_get_value(adj);
+	if((modified = g_object_get_data(G_OBJECT(adj), "modified")) != NULL)
 		*modified = TRUE;
 }
 
 /* Grab position. We implicitly *know* that the window in question is the main
- ** gentoo window, and therefore access it directly through the MainInfo. We
- ** then put the grabbed position into the editing copy of the WinDef, and update
- ** the spin buttons to match. Of course, we don't forget setting that pesky old
- ** modified-flag, either. :) What would people think?
- */
+** gentoo window, and therefore access it directly through the MainInfo. We
+** then put the grabbed position into the editing copy of the WinDef, and update
+** the spin buttons to match. Of course, we don't forget setting that pesky old
+** modified-flag, either. :) What would people think?
+*/
 static void evt_pos_grab_clicked(GtkWidget *wid, gpointer user)
 {
 	MainInfo *min;
 	WinDef *wd = user;
 
-	min = gtk_object_get_data(GTK_OBJECT(wid), "min");
+	min = g_object_get_data(G_OBJECT(wid), "min");
 	gdk_window_get_position(min->gui->window->window, &wd->x, &wd->y);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
-			GTK_OBJECT(wid), "spin1")), wd->x);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
-			GTK_OBJECT(wid), "spin2")), wd->y);
-	*(gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified") = TRUE;
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin1")), wd->x);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin2")), wd->y);
+	*(gboolean *) g_object_get_data(G_OBJECT(wid), "modified") = TRUE;
 }
 
 /* Grab size. Works only for main gentoo window. See comment for pos_grab above. */
@@ -187,13 +186,11 @@ static void evt_size_grab_clicked(GtkWidget *wid, gpointer user)
 	MainInfo *min;
 	WinDef *wd = user;
 
-	min = gtk_object_get_data(GTK_OBJECT(wid), "min");
+	min = g_object_get_data(G_OBJECT(wid), "min");
 	gdk_window_get_size(min->gui->window->window, &wd->width, &wd->height);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
-			GTK_OBJECT(wid), "spin1")), wd->width);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
-			GTK_OBJECT(wid), "spin2")), wd->height);
-	*(gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified") = TRUE;
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin1")), wd->width);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin2")), wd->height);
+	*(gboolean *) g_object_get_data(G_OBJECT(wid), "modified") = TRUE;
 }
 
 static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gboolean *modified)
@@ -208,25 +205,15 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 	frame = gtk_frame_new(_(ltext[type]));
 	vbox = gtk_vbox_new(FALSE, 0);
 	cbtn = gtk_check_button_new_with_label(_("Set on Open?"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type
-			== SUBFRAME_POSITION) ? win_window_pos_use_get(wi,
-			win->id) : win_window_size_use_get(wi, win->id));
-	gtk_object_set_user_data(GTK_OBJECT(cbtn), modified);
-	g_signal_connect(G_OBJECT(cbtn), "clicked", G_CALLBACK(
-			evt_boolean_clicked),
-			(type == SUBFRAME_POSITION) ? &win->pos_use
-					: &win->size_use);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type == SUBFRAME_POSITION) ? win_window_pos_use_get(wi, win->id) : win_window_size_use_get(wi, win->id));
+	g_object_set_data(G_OBJECT(cbtn), "modified", modified);
+	g_signal_connect(G_OBJECT(cbtn), "clicked", G_CALLBACK(evt_boolean_clicked), (type == SUBFRAME_POSITION) ? &win->pos_use : &win->size_use);
 	gtk_box_pack_start(GTK_BOX(vbox), cbtn, FALSE, FALSE, 0);
 
 	cbtn = gtk_check_button_new_with_label(_("Update on Close?"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type
-			== SUBFRAME_POSITION) ? win_window_pos_update_get(wi,
-			win->id) : win_window_size_update_get(wi, win->id));
-	gtk_object_set_user_data(GTK_OBJECT(cbtn), modified);
-	g_signal_connect(G_OBJECT(cbtn), "clicked", G_CALLBACK(
-			evt_boolean_clicked),
-			(type == SUBFRAME_POSITION) ? &win->pos_update
-					: &win->size_update);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type == SUBFRAME_POSITION) ? win_window_pos_update_get(wi, win->id) : win_window_size_update_get(wi, win->id));
+	g_object_set_data(G_OBJECT(cbtn), "modified", modified);
+	g_signal_connect(G_OBJECT(cbtn), "clicked", G_CALLBACK(evt_boolean_clicked), (type == SUBFRAME_POSITION) ? &win->pos_update : &win->size_update);
 	gtk_box_pack_start(GTK_BOX(vbox), cbtn, FALSE, FALSE, 0);
 
 	table = gtk_table_new(2, 3, FALSE);
@@ -240,37 +227,27 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 0, 0, 0, 0);
 	adj = gtk_adjustment_new(val1, -1.0, 65535.0, 1.0, 25.0, 0.0);
 	spin1 = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
-	gtk_object_set_user_data(GTK_OBJECT(spin1), modified);
-	g_signal_connect(G_OBJECT(spin1), "changed", G_CALLBACK(
-			evt_integer_changed),
-			type == SUBFRAME_POSITION ? &win->x : &win->width);
-	gtk_table_attach(GTK_TABLE(table), spin1, 1, 2, 0, 1, GTK_EXPAND
-			| GTK_FILL, 0, 0, 0);
+	g_object_set_data(G_OBJECT(adj), "modified", modified);
+	g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_integer_changed), type == SUBFRAME_POSITION ? &win->x : &win->width);
+	gtk_table_attach(GTK_TABLE(table), spin1, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(_(etext[2 * type + 1]));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 0, 0);
 	adj = gtk_adjustment_new(val2, -1.0, 65535.0, 1.0, 25.0, 0.0);
 	spin2 = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
-	gtk_object_set_user_data(GTK_OBJECT(spin2), modified);
-	g_signal_connect(G_OBJECT(spin2), "changed", G_CALLBACK(
-			evt_integer_changed),
-			type == SUBFRAME_POSITION ? &win->y : &win->height);
-	gtk_table_attach(GTK_TABLE(table), spin2, 1, 2, 1, 2, GTK_EXPAND
-			| GTK_FILL, 0, 0, 0);
-	if((type == SUBFRAME_POSITION && win->pos_grab) || (type
-			== SUBFRAME_SIZE && win->size_grab))
+	g_object_set_data(G_OBJECT(adj), "modified", modified);
+	g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_integer_changed), type == SUBFRAME_POSITION ? &win->y : &win->height);
+	gtk_table_attach(GTK_TABLE(table), spin2, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	if((type == SUBFRAME_POSITION && win->pos_grab) || (type == SUBFRAME_SIZE && win->size_grab))
 	{
 		GtkWidget *grab;
 
 		grab = gtk_button_new_with_label(_("Grab"));
 
-		gtk_object_set_data(GTK_OBJECT(grab), "min", wi->min);
-		gtk_object_set_data(GTK_OBJECT(grab), "spin1", spin1);
-		gtk_object_set_data(GTK_OBJECT(grab), "spin2", spin2);
-		gtk_object_set_data(GTK_OBJECT(grab), "modified", modified);
-		g_signal_connect(G_OBJECT(grab), "clicked", type
-				== SUBFRAME_POSITION ? G_CALLBACK(
-				evt_pos_grab_clicked) : G_CALLBACK(
-				evt_size_grab_clicked), win);
+		g_object_set_data(G_OBJECT(grab), "min", wi->min);
+		g_object_set_data(G_OBJECT(grab), "spin1", spin1);
+		g_object_set_data(G_OBJECT(grab), "spin2", spin2);
+		g_object_set_data(G_OBJECT(grab), "modified", modified);
+		g_signal_connect(G_OBJECT(grab), "clicked", type == SUBFRAME_POSITION ? G_CALLBACK(evt_pos_grab_clicked) : G_CALLBACK(evt_size_grab_clicked), win);
 		gtk_table_attach(GTK_TABLE(table), grab, 2, 3, 0, 2, 0, 0, 0, 0);
 	}
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
@@ -292,8 +269,7 @@ GtkWidget * win_wininfo_build(const WinInfo *wi, gboolean *modified)
 	{
 		win = iter->data;
 
-		g_snprintf(tmp, sizeof tmp, "%c%s", toupper(*win->label),
-				win->label + 1);
+		g_snprintf(tmp, sizeof tmp, "%c%s", toupper(*win->label), win->label + 1);
 		frame = gtk_frame_new(tmp);
 		hbox = gtk_hbox_new(FALSE, 0);
 		sframe = subframe_build(wi, win, SUBFRAME_POSITION, modified);
@@ -595,6 +571,19 @@ void win_window_destroy(WinInfo *wi, guint32 id)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2010-12-12 -	Rewritten to not use deprecated (and crazy-seeming) GDK calls. */
+static void window_set_icon(GtkWindow *win)
+{
+	GdkPixbuf	*pbuf;
+
+	if((pbuf = gdk_pixbuf_new_from_xpm_data(icon_gentoo_small_xpm)) != NULL)
+	{
+		gtk_window_set_default_icon(pbuf);
+		gtk_window_set_icon_name(win, PACKAGE);
+		g_object_unref(G_OBJECT(pbuf));	/* I believe GTK+ owns this, now. */
+	}
+}
+
 /* 1999-12-23 -	Open the window described by <key> in <wi>. The window will not be shown yet. */
 GtkWidget * win_window_open(const WinInfo *wi, guint32 id)
 {
@@ -603,9 +592,6 @@ GtkWidget * win_window_open(const WinInfo *wi, guint32 id)
 	if((wd = window_find(wi, id)) != NULL)
 	{
 		GtkWidget *wid = NULL;
-		gint old_errno;
-
-		old_errno = errno;
 
 		switch (wd->type)
 		{
@@ -617,50 +603,50 @@ GtkWidget * win_window_open(const WinInfo *wi, guint32 id)
 			wid = gtk_window_new(GTK_WINDOW_POPUP);
 			break;
 		case WIN_TYPE_COMPLEX_DIALOG:
-			wid = gtk_dialog_new();
+			wid = win_dialog_open(wi->min->gui->window);
 			break;
 		}
-		gtk_object_set_data(GTK_OBJECT(wid), "wininfo", (gpointer) wi);
-		gtk_object_set_data(GTK_OBJECT(wid), "windef", (gpointer) wd);
+		g_object_set_data(G_OBJECT(wid), "wininfo", (gpointer) wi);
+		g_object_set_data(G_OBJECT(wid), "windef", (gpointer) wd);
 		if(!wd->pos_use)
 			gtk_window_set_position(GTK_WINDOW(wid), GTK_WIN_POS_MOUSE);
 		gtk_widget_realize(wid);
-		if(wi->min->gui != NULL && id != WIN_MAIN)
+		if(id == WIN_MAIN)
+			window_set_icon(GTK_WINDOW(wid));
+		else if(id != WIN_MAIN && id != WIN_TYPE_COMPLEX_DIALOG && wi->min->gui != NULL)
 			gtk_window_set_transient_for(GTK_WINDOW(wid), GTK_WINDOW(wi->min->gui->window));
 		if(wd->title != NULL)
 			win_window_set_title(wid, wd->title);
-		errno = old_errno;
 		return wid;
 	}
 	return NULL;
 }
 
 /* 2000-02-19 -	Refresh link between <wi> and <window>. Handy after <wi> has been replaced
- **		while window was open, as happens in the config window case.
- */
+**		while window was open, as happens in the config window case.
+*/
 void win_window_relink(const WinInfo *wi, guint32 id, GtkWidget *window)
 {
 	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 	{
-		gtk_object_set_data(GTK_OBJECT(window), "wininfo", (gpointer) wi);
-		gtk_object_set_data(GTK_OBJECT(window), "windef", wd);
+		g_object_set_data(G_OBJECT(window), "wininfo", (gpointer) wi);
+		g_object_set_data(G_OBJECT(window), "windef", wd);
 	}
 }
 
 /* 2002-08-10 -	Set title of a window. Since it seems to break if locale is set, we simply
- **		un-set the locale temporarily. Hopefully, noone sets window titles in inner
- **		loops around here.
- ** NOTE NOTE	This works for *all* windows, not just those created through this module.
- **		That might surprise, but it felt more sensical to stick it in here anyway.
- */
+**		un-set the locale temporarily. Hopefully, noone sets window titles in inner
+**		loops around here.
+** NOTE NOTE	This works for *all* windows, not just those created through this module.
+**		That might surprise, but it felt more sensical to stick it in here anyway.
+*/
 void win_window_set_title(GtkWidget *win, const gchar *title)
 {
 #if defined ENABLE_NLS
 	gchar obuf[256]; /* Buffer, subsequent setlocale() call kills it otherwise. */
 	const gchar *op;
-	int old_errno = errno; /* Buffer the errno, so any errors here don't leak out. */
 
 	/* Get current locale, before slamming "C" in there. */
 	if((op = setlocale(LC_ALL, NULL)) != NULL)
@@ -677,36 +663,46 @@ void win_window_set_title(GtkWidget *win, const gchar *title)
 
 #if defined ENABLE_NLS
 	if(op != NULL)
-	setlocale(LC_ALL, obuf);
-	errno = old_errno; /* Restore the errno, ignoring any errors that might have happened here. */
+		setlocale(LC_ALL, obuf);
 #endif
 }
 
 /* 1999-12-23 -	Use this rather than gtk_widget_show() when you want to the window to appear, since
- **		this gives us a chance to set the window's size and position according to preference.
- */
+**		this gives us a chance to set the window's size and position according to preference.
+*/
 void win_window_show(GtkWidget *window)
 {
 	WinInfo	*wi;
 	WinDef	*wd;
-	gint	old_errno;
 
+#if GTK_CHECK_VERSION(2, 20, 0)
+	if(gtk_widget_get_visible(window))
+		return;
+#else
 	if(GTK_WIDGET_VISIBLE(window))
 		return;
+#endif
 
-	wi = gtk_object_get_data(GTK_OBJECT(window), "wininfo");
-	wd = gtk_object_get_data(GTK_OBJECT(window), "windef");
+	wi = g_object_get_data(G_OBJECT(window), "wininfo");
+	wd = g_object_get_data(G_OBJECT(window), "windef");
 	if((wd != NULL) && wd->size_use && wd->width >= 0)
 		gtk_window_set_default_size(GTK_WINDOW(window), wd->width, wd->height);
+	/* For the main window, make sure to make it possible to shrink very much. */
+	if(wd != NULL && wd->id == WIN_MAIN)
+	{
+		GdkGeometry	geo;
+
+		geo.min_width = 10;
+		geo.min_height = 10;
+		gtk_window_set_geometry_hints(GTK_WINDOW(window), window, &geo, GDK_HINT_MIN_SIZE);
+	}
 
 	/* This causes the window to appear at some more or less random position. We then reposition
 	** it to the configured place. This is visually distracting, unattractive, and almost s*cks,
 	** but unfortunately I haven't been able to figure out how to fix it. If you move the show()
 	** call to below the if, it works, but not all the time. :( Tips welcome.
 	*/
-	old_errno = errno;
 	gtk_widget_show_all(window);
-	errno = old_errno;
 
 	/* If user wants to set position, check where it ended up and move it home. */
 	if((wd != NULL) && wd->pos_use && wd->x >= 0)
@@ -726,8 +722,8 @@ void win_window_show(GtkWidget *window)
 }
 
 /* 2000-03-04 -	If <a_new> is different from *<a>, update *<a> to <a_new> and return TRUE, else
- **		return FALSE. Likewise for <b>, whatever that would mean.
- */
+**		return FALSE. Likewise for <b>, whatever that would mean.
+*/
 static gboolean update_pair(gint *a, gint *b, gint a_new, gint b_new)
 {
 	if((*a != a_new) || (*b != b_new))
@@ -740,13 +736,13 @@ static gboolean update_pair(gint *a, gint *b, gint a_new, gint b_new)
 }
 
 /* 2000-03-18 -	Do an update for the given window, changing the config as needed. Returns TRUE
- **		if the config was indeed changed.
- */
+**		if the config was indeed changed.
+*/
 gboolean win_window_update(GtkWidget *win)
 {
 	WinDef *wd;
 
-	if((wd = gtk_object_get_data(GTK_OBJECT(win), "windef")) != NULL)
+	if((wd = g_object_get_data(G_OBJECT(win), "windef")) != NULL)
 	{
 		gboolean ret = FALSE;
 		gint ta, tb;
@@ -767,16 +763,16 @@ gboolean win_window_update(GtkWidget *win)
 }
 
 /* 1999-12-23 -	Close the given window. Use this rather than a direct gtk_widget_destroy(), since
- **		we need a chance to update the position and size fields if needed. Also, if the
- **		window is classified as "persistent", we don't actually destroy it, but only hide
- **		it. Returns TRUE if the config was changed, i.e. if one of the update flags were
- **		set AND the relevant property actually had changed.
- */
+**		we need a chance to update the position and size fields if needed. Also, if the
+**		window is classified as "persistent", we don't actually destroy it, but only hide
+**		it. Returns TRUE if the config was changed, i.e. if one of the update flags were
+**		set AND the relevant property actually had changed.
+*/
 gboolean win_window_close(GtkWidget *win)
 {
 	WinDef *wd;
 
-	if((wd = gtk_object_get_data(GTK_OBJECT(win), "windef")) != NULL)
+	if((wd = g_object_get_data(G_OBJECT(win), "windef")) != NULL)
 	{
 		gboolean ret;
 
@@ -790,3 +786,18 @@ gboolean win_window_close(GtkWidget *win)
 	gtk_widget_destroy(win);
 	return FALSE;
 }
+
+/* ----------------------------------------------------------------------------------------- */
+
+/* 2010-12-12 -	New single way of opening a dialog, that tries to make sure it gets the proper
+**		linkage to the main window. This shares the icon, among (possibly) other things.
+*/
+GtkWidget * win_dialog_open(GtkWidget *main_window)
+{
+	GtkWidget	*wid = gtk_dialog_new();
+
+	if(main_window != NULL)
+		gtk_window_set_transient_for(GTK_WINDOW(wid), GTK_WINDOW(main_window));
+
+	return wid;
+}
