@@ -269,7 +269,7 @@ static void put_string(FILE *out, const gchar *str)
 		{
 			if(here == esc_seq[i].equiv)
 			{
-				fprintf(out, esc_seq[i].sequence);
+				fprintf(out, "%s", esc_seq[i].sequence);
 				break;
 			}
 		}
@@ -401,7 +401,7 @@ gboolean xml_get_text_copy(const XmlNode *node, const gchar *name, gchar *str, g
 	if((data = get_node(node, name, XVT_TEXT)) != NULL)
 	{
 		if(str != NULL)
-			stu_strncpy(str, data->data.text.value->str, max);
+			g_strlcpy(str, data->data.text.value->str, max);
 		return TRUE;
 	}
 	return FALSE;
@@ -502,7 +502,7 @@ XmlNode * xml_node_new(const gchar *name)
 	if((node = g_malloc(sizeof *node)) != NULL)
 	{
 		if(name != NULL)
-			stu_strncpy(node->name, name, sizeof node->name);
+			g_strlcpy(node->name, name, sizeof node->name);
 		else
 			node->name[0] = '\0';
 		node->data.type = XVT_NONE;
